*!* Programme / Procdure : installDepotFox.prg
*!* Auteur : Michel LVY
*!* Date: 25/07/2017 14:51:21
*!*
*!* Description: cration de la base de donnes SQL Server de dpot historis des binaires fox
*!*
*!*Paramtres attendus :
* la chaine de connexion vers l'instance SQL Server
* le nom de la base de donnes que vous voulez crer dans cette instance SQL
* le nom du dossier FileStream pour l'accs Windows
* (c'est dans ce dossier que vous trouverez le sous-dossier "depotfox" dans lequel vous copierez les fichiers fox  archiver pour historique)
*
*!* Spcificits de la version :
*!*	25/07/2017	1.0 	fonctionnelle
*!*	05/08/2017	1.1 	ajout du GRANT dans le createDatabase, suppression de la vrification de l'accs au dossier partag
*!* 07/08/2017 	2.0 	Split de la database pour isoler foxdepot
*!* 31/08/2017	2.1		Implmentation des projets hirarchiss
*!*	09/09/2017	2.1		modification de la vue sur les tables/projets
*!* 08/11/2017	2.2		ajout du traitement des prg, mpr, qpr
*!* 09/12/2017	3.0		Traitement de tous les types de fichiers (non fox)
*!* 19/12/2017	3.0		modification de structure de la table Version
*=============================================================
LPARAMETERS lcCnxString AS STRING, lcDbName AS STRING, lcPathName AS STRING
PRIVATE lnCnxHandle AS INTEGER
PUBLIC	gcVirtualPath AS STRING	&& le rpertoire virtuel qui sera cr

STORE "" TO m.gcVirtualPath

* vrifier les paramtres et vrifier la qualit des paramtres
IF PCOUNT() <> 3 OR (VARTYPE(m.lcCnxString)<>'C' AND VARTYPE(m.lcDbName)<>'C' AND VARTYPE(m.lcPathName)<>'C')
	MESSAGEBOX('Vous devez passer les 3 paramtres demands')
	RETURN
ELSE
	* tester la chaine de connexion, si OK conserver le handle
	LOCAL lnHcnx AS INTEGER
	m.lnHcnx=SQLSTRINGCONNECT(m.lcCnxString)
	IF m.lnHcnx > 0
		STORE m.lnHcnx TO m.lnCnxHandle
	ELSE
		MESSAGEBOX("chaine de connexion invalide")
		RETURN
	ENDIF
ENDIF

LOCAL llResult AS Boolean
* vrifier les prrequis SQL
m.llResult=VerifPrerequis()
IF m.llResult=.F.
	MESSAGEBOX("Vrifiez les prrequis de votre instance SQL")
	RETURN
ENDIF

IF m.llResult
	* crer la database
	m.lcdbname=ALLTRIM(m.lcdbname)
	WAIT WINDOW "Cration de la database" NOWAIT
	LOCAL llOKcreateDB AS Boolean
	llOKcreateDB = CreateDatabase(m.lcDbName, m.lcPathName)
	m.llResult=m.llOKcreateDB
ENDIF

IF m.llResult
	*  partir d'ici, on peut intgrer le nom de la database dans la chaine de connexion, donc on ferme et rouvre une connexion
	LOCAL lcNewCnxString AS STRING
	m.lcNewCnxString=STRTRAN(m.lcCnxString, "master", ALLTRIM(m.lcDbName),1,1,1)
	LOCAL lnHcnx AS INTEGER
	m.lnHcnx=SQLSTRINGCONNECT(m.lcNewCnxString)
	IF 	m.lnHcnx > 0
		SQLDISCONNECT(m.lnCnxHandle)
		STORE m.lnHcnx TO m.lnCnxHandle
	ELSE
		MESSAGEBOX("Impossible de se connecter  la nouvelle base de donnes")
		RETURN
	ENDIF

	* crer les tables
	WAIT WINDOW "Cration des tables" NOWAIT
	LOCAL llOKcreateTables AS Boolean
	llOKcreateTables = CreateTables(m.lcDbName)
	m.llResult=m.llOKcreateTables
ENDIF

IF m.llResult
	* crer les index
	WAIT WINDOW "Cration des index" NOWAIT
	LOCAL llOKcreateIndexes AS Boolean
	llOKcreateIndexes = CreateIndexes(m.lcDbName)
	m.llResult=m.llOKcreateIndexes
ENDIF

IF m.llResult
	* crer les FK
	WAIT WINDOW "Cration des Foreign Keys" NOWAIT
	LOCAL llOKcreateFK AS Boolean
	llOKcreateFK = CreateFK(m.lcDbName)
	m.llResult=m.llOKcreateFK
ENDIF

IF m.llResult
	* crer les procdures
	WAIT WINDOW "Cration des Procdures Stockes" NOWAIT
	LOCAL llOKcreateProc AS Boolean
	llOKcreateProc = CreateProc(m.lcDbName)
	m.llResult=m.llOKcreateProc
ENDIF

IF m.llResult
	* crer les vues
	WAIT WINDOW "Cration des Vues" NOWAIT
	LOCAL llOKcreateViews AS Boolean
	llOKcreateViews = CreateViews(m.lcDbName)
	m.llResult=m.llOKcreateViews
ENDIF


****************************************************************
FUNCTION VerifPrerequis AS Boolean
	LOCAL llResult AS Boolean
	* si le n de version est <13, on n'a pas un SQL 2016 ou plus, on sort
	IF GetNumVersion() <13
		m.llResult = .F.
		MESSAGEBOX("DepotFox ne peut fonctionner que sur du SQL Server 2016 ou suprieur")
		SQLDISCONNECT(0)
		RETURN
	ENDIF

	* si FileStream n'est pas activ sur l'instance, ou activ  un niveau insuffisant, on sort
	IF GetFileStreamLevel() < 3
		m.llResult = .F.
		MESSAGEBOX("Filestream doit tre activ en mode Full pour pouvoir utiliser DepotFox")
		SQLDISCONNECT(0)
		RETURN
	ENDIF

	m.llResult=.T.
	RETURN m.llResult
ENDFUNC && VerifPrerequis

*****************************************************
FUNCTION GetNumVersion AS INTEGER
	* DECLARE @numversion int= CONVERT(int,SERVERPROPERTY('ProductMajorVersion'))
	LOCAL NumVersion AS INTEGER
	m.NumVersion=0
	TRY
		SQLEXEC(m.lnCnxHandle," SELECT CONVERT(int,SERVERPROPERTY('ProductMajorVersion')) AS Numero","curVer")
		STORE curver.numero TO m.NumVersion
	CATCH

	FINALLY
		USE IN SELECT("curver")
	ENDTRY
	RETURN m.NumVersion
ENDFUNC && GetNumVersion

***************************************************
FUNCTION GetFileStreamLevel AS INTEGER
	* DECLARE @niveauFS int = CONVERT(int, SERVERPROPERTY('FilestreamConfiguredLevel'))
	LOCAL niveauFS AS INTEGER
	m.niveauFS=0
	TRY
		SQLEXEC(m.lnCnxHandle, " SELECT CONVERT(int, SERVERPROPERTY('FilestreamConfiguredLevel')) AS Niveau", "curFS")
		STORE curfs.niveau TO m.niveauFS
	CATCH

	FINALLY
		USE IN SELECT("curfs")
	ENDTRY

	RETURN m.niveauFS
ENDFUNC && GetFileStreamLevel

****************************************************************
FUNCTION CreateDatabase AS Boolean
	LPARAMETERS dbName, PathName
	LOCAL llResult AS Boolean

	* il faut crer 2 databases, une conserve le nom, l'autre le nom+Archive

	LOCAL lcDataPath AS STRING, lcLogPath AS STRING, lcUser AS STRING, lcDbArchiveName AS STRING
	m.lcDbArchiveName=ALLTRIM(m.dbName)+"_archive"

	TRY
		SQLEXEC(m.lnCnxHandle,;
			" SELECT CONVERT(sysname, SERVERPROPERTY('InstanceDefaultDataPath')) AS datapath, CONVERT(sysname, SERVERPROPERTY('InstanceDefaultLogPath')) AS logpath , SUSER_NAME() as usercnx",;
			"curPath")
		STORE ALLTRIM(curpath.datapath) TO m.lcDataPath
		STORE ALLTRIM(curpath.logpath) TO m.lcLogPath
		STORE ALLTRIM(curpath.usercnx) TO m.lcUser
		* attention, si la connexion est TrustedConnexion, la valeur rendue ici est incorrecte, il faut la reprendre par l'environnement
		m.lcUser=IIF("MICROSOFTACCOUNT" $ UPPER(m.lcUser), GETENV("USERDOMAIN")+"\"+GETENV("USERNAME"), m.lcUser)

		* le SQLCREATE de la database de base
		TEXT TO lcDBcreate NOSHOW TEXTMERGE

-- la database destine  la filetable

IF NOT EXISTS (SELECT name FROM sys.databases WHERE name = N'<<m.dbname>>')
BEGIN
   CREATE DATABASE [<<m.dbname>>]
    ON PRIMARY
	   (
	   NAME = N'<<m.dbname>>_data'
	    , FILENAME = N'<<m.lcDataPath>><<m.dbname>>_data.mdf'
	    , SIZE = 150MB
	    , MAXSIZE = UNLIMITED
	    , FILEGROWTH = 50MB
	    ) ,
    FILEGROUP [FileStream<<m.dbname>>] CONTAINS FILESTREAM DEFAULT
	   (
	   NAME = N'<<m.dbname>>'
	   ,FILENAME = N'<<m.lcDataPath>><<m.dbname>>_filestream'
    , MAXSIZE = UNLIMITED
	   )
    LOG ON
	   (
	   NAME = N'<<m.dbname>>_log'
	    , FILENAME = N'<<m.lcLogPath>><<m.dbname>>_log.ldf'
	    , SIZE = 25MB
	    , MAXSIZE = 250MB
	    , FILEGROWTH = 50MB
	    ) ;
END ;

	ALTER DATABASE [<<m.dbname>>] SET FILESTREAM( NON_TRANSACTED_ACCESS = FULL, DIRECTORY_NAME = N'<<m.PathName>>' ) ;
	ALTER DATABASE [<<m.dbname>>] SET ANSI_NULLS ON ;
	ALTER DATABASE [<<m.dbname>>] SET ANSI_PADDING ON ;
	ALTER DATABASE [<<m.dbname>>] SET ANSI_WARNINGS ON ;
	ALTER DATABASE [<<m.dbname>>] SET ARITHABORT ON ;
	ALTER DATABASE [<<m.dbname>>] SET AUTO_CLOSE OFF ;
	ALTER DATABASE [<<m.dbname>>] SET AUTO_SHRINK OFF ;
	ALTER DATABASE [<<m.dbname>>] SET AUTO_UPDATE_STATISTICS ON ;
	ALTER DATABASE [<<m.dbname>>] SET CURSOR_CLOSE_ON_COMMIT OFF ;
	ALTER DATABASE [<<m.dbname>>] SET CURSOR_DEFAULT  GLOBAL ;
	ALTER DATABASE [<<m.dbname>>] SET CONCAT_NULL_YIELDS_NULL ON ;
	ALTER DATABASE [<<m.dbname>>] SET RECOVERY SIMPLE ;
	ALTER DATABASE [<<m.dbname>>] SET MULTI_USER ;

	GRANT ALL ON <<m.dbname>> TO [<<m.lcUser>>] ;

-- la database destine  l'archivage historis

IF NOT EXISTS (SELECT name FROM sys.databases WHERE name = N'<<m.lcDbArchiveName>>')
BEGIN
   CREATE DATABASE [<<m.lcDbArchiveName>>]
    ON PRIMARY
	   (
	   NAME = N'<<m.lcDbArchiveName>>_data'
	    , FILENAME = N'<<m.lcDataPath>><<m.lcDbArchiveName>>_data.mdf'
	    , SIZE = 150MB
	    , MAXSIZE = UNLIMITED
	    , FILEGROWTH = 50MB
	    )
    LOG ON
	   (
	   NAME = N'<<m.lcDbArchiveName>>_log'
	    , FILENAME = N'<<m.lcLogPath>><<m.lcDbArchiveName>>_log.ldf'
	    , SIZE = 25MB
	    , MAXSIZE = 250MB
	    , FILEGROWTH = 50MB
	    ) ;
END ;

	ALTER DATABASE [<<m.lcDbArchiveName>>] SET ANSI_NULLS ON ;
	ALTER DATABASE [<<m.lcDbArchiveName>>] SET ANSI_PADDING ON ;
	ALTER DATABASE [<<m.lcDbArchiveName>>] SET ANSI_WARNINGS ON ;
	ALTER DATABASE [<<m.lcDbArchiveName>>] SET ARITHABORT ON ;
	ALTER DATABASE [<<m.lcDbArchiveName>>] SET AUTO_CLOSE OFF ;
	ALTER DATABASE [<<m.lcDbArchiveName>>] SET AUTO_SHRINK OFF ;
	ALTER DATABASE [<<m.lcDbArchiveName>>] SET AUTO_UPDATE_STATISTICS ON ;
	ALTER DATABASE [<<m.lcDbArchiveName>>] SET CURSOR_CLOSE_ON_COMMIT OFF ;
	ALTER DATABASE [<<m.lcDbArchiveName>>] SET CURSOR_DEFAULT  GLOBAL ;
	ALTER DATABASE [<<m.lcDbArchiveName>>] SET CONCAT_NULL_YIELDS_NULL ON ;
	ALTER DATABASE [<<m.lcDbArchiveName>>] SET RECOVERY SIMPLE ;
	ALTER DATABASE [<<m.lcDbArchiveName>>] SET MULTI_USER ;

	GRANT ALL ON <<m.lcDbArchiveName>> TO [<<m.lcUser>>] ;


		ENDTEXT

		SQLEXEC(m.lnCnxHandle, m.lcdbcreate,"curresult")

	CATCH
		=AERROR(aErraray)
		MESSAGEBOX("plop Create databases")
	FINALLY
		* vrification 1
		TEXT TO lcSqlVerif NOSHOW TEXTMERGE
	SELECT name as nom FROM sys.databases WHERE name = N'<<m.dbname>>'
		ENDTEXT

		SQLEXEC(m.lnCnxHandle, m.lcSqlVerif ,"curresult")
		IF UPPER(ALLTRIM(curresult.nom))==UPPER(m.dbName)
			* vrification 2
			TEXT TO lcSqlVerif NOSHOW TEXTMERGE
	SELECT name as nom FROM sys.databases WHERE name = N'<<m.lcDbArchiveName>>'
			ENDTEXT
			SQLEXEC(m.lnCnxHandle, m.lcSqlVerif ,"curresult")
			IF UPPER(ALLTRIM(curresult.nom))==UPPER(m.lcDbArchiveName)
				m.llResult=.T.
			ENDIF

		ELSE
			m.llResult=.F.
		ENDIF

		USE IN SELECT("curpath")
		USE IN SELECT("curresult")
	ENDTRY

	WAIT CLEAR
	RETURN m.llResult
ENDFUNC && CreateDatabase

*****************************************************
FUNCTION CreateTables AS Boolean
	LPARAMETERS dbName
	LOCAL llResult AS Boolean

	* il faut rpartir les tables entre les deux databases (depotfox dans dbname)
	LOCAL lcDbArchiveName AS STRING
	m.lcDbArchiveName=ALLTRIM(m.dbName)+"_archive"

	TRY
		TEXT TO lcTablesCreate NOSHOW TEXTMERGE
	-- depotfox
	USE [<<m.dbname>>] ;
	/****** Object:  Table [dbo].[depotpourfox]    Script Date: 19/07/2017 12:03:29 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	SET ARITHABORT ON
	;
	IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[depotpourfox]') AND type in (N'U'))
	BEGIN
		CREATE TABLE [dbo].[depotpourfox] AS FILETABLE ON [PRIMARY] FILESTREAM_ON [FileStream<<m.dbname>>]
		WITH
		(
		FILETABLE_DIRECTORY = N'depotfox', FILETABLE_COLLATE_FILENAME = French_CI_AS
		)
	END
	;
	/****** Object:  Table [dbo].[TYPE_FICHIERS]    Script Date: 03/12/2017 15:44:41 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	SET ARITHABORT ON
	;	
	IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[TYPE_FICHIERS]') AND type in (N'U'))
	BEGIN
	CREATE TABLE [dbo].[TYPE_FICHIERS](
		[PK] [smallint] IDENTITY(1,1) NOT NULL,
		[TYP_nom] [varchar](35) NOT NULL,
		[TYP_ext] [varchar](5) NOT NULL,
		[TYP_actif] [bit] NOT NULL,
	 CONSTRAINT [PK_TYPE_FICHIERS] PRIMARY KEY CLUSTERED 
	(
		[PK] ASC
	)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY],
	 CONSTRAINT [UQ_TYP_ext] UNIQUE NONCLUSTERED 
	(
		[TYP_ext] ASC
	)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
	) ON [PRIMARY]
	END	
	;	
	GRANT EXECUTE, SELECT, INSERT, UPDATE, DELETE ON SCHEMA ::dbo TO public;

	-- les tables avec historique
	USE [<<m.lcDbArchiveName>>] ;
	/****** Object:  Table [dbo].[foxtable_historique]    Script Date: 19/07/2017 12:03:29 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[foxtable_historique]') AND type in (N'U'))
	BEGIN
		CREATE TABLE [dbo].[foxtable_historique](
			[PK] [int] NOT NULL,
			[fox_nom_table] [varchar](128) NOT NULL,
			[fox_data] [varbinary](max) NULL,
			[fox_index] [varbinary](max) NULL,
			[fox_memo] [varbinary](max) NULL,
			[Debut] [datetime2](7) NOT NULL,
			[Fin] [datetime2](7) NOT NULL,
			[fox_typetable] [char](10) NULL,
			[fox_chemin] [nvarchar](max) NULL
		) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
	END
	;
	/****** Object:  Index [ix_foxtable_historique]    Script Date: 19/07/2017 12:03:29 ******/
	IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[foxtable_historique]') AND name = N'ix_foxtable_historique')
		CREATE CLUSTERED INDEX [ix_foxtable_historique] ON [dbo].[foxtable_historique]
		(
			[Fin] ASC,
			[Debut] ASC
		)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
	;
	/****** Object:  Table [dbo].[foxtable]    Script Date: 19/07/2017 12:03:29 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[foxtable]') AND type in (N'U'))
	BEGIN
		CREATE TABLE [dbo].[foxtable](
			[PK] [int] IDENTITY(1,1) NOT NULL,
			[fox_nom_table] [varchar](128) NOT NULL,
			[fox_data] [varbinary](max) NULL,
			[fox_index] [varbinary](max) NULL,
			[fox_memo] [varbinary](max) NULL,
			[Debut] [datetime2](7) GENERATED ALWAYS AS ROW START NOT NULL,
			[Fin] [datetime2](7) GENERATED ALWAYS AS ROW END NOT NULL,
			[fox_typetable] [char](10) NULL,
			[fox_chemin] [nvarchar](max) NULL,
		 CONSTRAINT [PK_foxtable] PRIMARY KEY CLUSTERED
		(
			[PK] ASC
		)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY],
		 CONSTRAINT [UQ_PK_debut_fin] UNIQUE NONCLUSTERED 
		(
			[PK] ASC,
			[Debut] ASC,
			[Fin] ASC
		)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY],		
			PERIOD FOR SYSTEM_TIME ([Debut], [Fin])
		) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
		WITH
		(
		SYSTEM_VERSIONING = ON ( HISTORY_TABLE = [dbo].[foxtable_historique] )
		)
	END
	;
	
	/****** Object:  Table [dbo].[fichiers_historique]    Script Date: 03/12/2017 15:48:02 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[fichiers_historique]') AND type in (N'U'))
	BEGIN
	CREATE TABLE [dbo].[fichiers_historique](
		[PK] [int] NOT NULL,
		[fichier_nom] [varchar](128) NOT NULL,
		[fichier_data] [varbinary](max) NULL,
		[signature_nom] [varchar](128) NULL,
		[signature_data] [varbinary](max) NULL,
		[Debut] [datetime2](7) NOT NULL,
		[Fin] [datetime2](7) NOT NULL,
		[fichier_type] [varchar](35) NULL,
		[fichier_chemin] [nvarchar](max) NULL
	) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
	END
	;
	/****** Object:  Index [ix_fichier_historique]    Script Date: 03/12/2017 15:48:02 ******/
	IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[fichiers_historique]') AND name = N'ix_fichier_historique')
	CREATE CLUSTERED INDEX [ix_fichier_historique] ON [dbo].[fichiers_historique]
	(
		[Fin] ASC,
		[Debut] ASC
	)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
	;
	/****** Object:  Table [dbo].[fichiers]    Script Date: 03/12/2017 15:48:02 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[fichiers]') AND type in (N'U'))
	BEGIN
	CREATE TABLE [dbo].[fichiers](
		[PK] [int] IDENTITY(1,1) NOT NULL,
		[fichier_nom] [varchar](128) NOT NULL,
		[fichier_data] [varbinary](max) NULL,
		[signature_nom] [varchar](128) NULL,
		[signature_data] [varbinary](max) NULL,
		[Debut] [datetime2](7) GENERATED ALWAYS AS ROW START NOT NULL,
		[Fin] [datetime2](7) GENERATED ALWAYS AS ROW END NOT NULL,
		[fichier_type] [varchar](35) NULL,
		[fichier_chemin] [nvarchar](max) NULL,
	 CONSTRAINT [PK_fichiers] PRIMARY KEY CLUSTERED 
	(
		[PK] ASC
	)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY],
	 CONSTRAINT [UQ_PK_debut_fin_2] UNIQUE NONCLUSTERED 
	(
		[PK] ASC,
		[Debut] ASC,
		[Fin] ASC
	)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY],
		PERIOD FOR SYSTEM_TIME ([Debut], [Fin])
	) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
	WITH
	(
	SYSTEM_VERSIONING = ON ( HISTORY_TABLE = [dbo].[fichiers_historique] )
	)
	END
	;
	
	
	-- les autres tables

	/****** Object:  Table [dbo].[FoxTable_Fichiers_Projet]    Script Date: 18/01/2018 10:47:14 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[FoxTable_Fichiers_Projet]') AND type in (N'U'))
	BEGIN
		CREATE TABLE [dbo].[FoxTable_Fichiers_Projet](
			[FK_FoxTable] [int] NULL,
			[FK_Fichiers] [int] NULL,
			[FK_Projet] [int] NOT NULL,
			[PK] [smallint] IDENTITY(1,1) NOT NULL,
		 CONSTRAINT [PK_FoxTable_Fichiers_Projet] PRIMARY KEY CLUSTERED
		(
			[PK] ASC
		)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
		) ON [PRIMARY]
	END
	;
		/** contrainte de non null **/
		IF NOT EXISTS (SELECT * FROM sys.check_constraints WHERE object_id = OBJECT_ID(N'[dbo].[Ck_Projet_Foxoufich_Null]') AND parent_object_id = OBJECT_ID(N'[dbo].[FoxTable_Fichiers_Projet]'))
		BEGIN
		ALTER TABLE [dbo].[FoxTable_Fichiers_Projet]  WITH CHECK ADD  CONSTRAINT [Ck_Projet_Foxoufich_Null] CHECK  (([Fk_Foxtable] IS NULL AND [Fk_Fichiers] IS NOT NULL OR [Fk_Foxtable] IS NOT NULL AND [Fk_Fichiers] IS NULL))
		END
		;			

/****** Object:  Table [dbo].[Projet]    Script Date: 28/08/2017 15:17:25 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Projet]') AND type in (N'U'))
	BEGIN
	CREATE TABLE [dbo].[Projet](
		[PK] [int] IDENTITY(1,1) NOT NULL,
		[nom_projet] [char](50) NOT NULL,
		[Commentaire] [varchar](4000) NULL,
		[niveauID] [hierarchyid] NOT NULL,
		[niveau]  AS (CONVERT([varchar](254),[niveauID])) PERSISTED,
		[niveaucomplet]  AS (CONVERT([varchar](4000),[niveauID])) PERSISTED,
	 CONSTRAINT [PK_Projet] PRIMARY KEY CLUSTERED
	(
		[PK] ASC
	)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY],
	 CONSTRAINT [UQ_Projet_niveauID] UNIQUE NONCLUSTERED
	(
		[niveauID] ASC
	)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
	) ON [PRIMARY]
	END
;
/****** Object:  Table [dbo].[Version]    Script Date: 24/09/2017 18:02:53 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Version]') AND type in (N'U'))
	BEGIN
	CREATE TABLE [dbo].[Version](
		[PK] [int] IDENTITY(1,1) NOT NULL,
		[numeroID] [hierarchyid] NOT NULL,
		[numero]  AS (CONVERT([varchar](254),RIGHT(LEFT(REPLACE(CONVERT([varchar](254),[numeroid]),'/','.'),LEN(CONVERT([varchar](254),[numeroid]))-(1)),LEN(CONVERT([varchar](254),[numeroid]))-(2)))) PERSISTED,
		[FK_foxtable] [int] NULL,
		[FK_fichiers] [int] NULL,
		[debut] [datetime2](7) NOT NULL,
	 CONSTRAINT [PK_Version] PRIMARY KEY CLUSTERED 
	(
		[PK] ASC
	)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
	) ON [PRIMARY]
	END
	;
		/** valeur par defaut **/
		IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DF_Version_debut]') AND type = 'D')
		BEGIN
		ALTER TABLE [dbo].[Version] ADD  CONSTRAINT [DF_Version_debut]  DEFAULT ('1900-01-01T00:00:00.0000000') FOR [debut]
		END 
		;
		
		/** contrainte de non null **/
		IF NOT EXISTS (SELECT * FROM sys.check_constraints WHERE object_id = OBJECT_ID(N'[dbo].[Ck_Version_Foxoufich_Null]') AND parent_object_id = OBJECT_ID(N'[dbo].[Version]'))
		BEGIN
		ALTER TABLE [dbo].[Version]  WITH CHECK ADD  CONSTRAINT [Ck_Version_Foxoufich_Null] CHECK  (([Fk_Foxtable] IS NULL AND [Fk_Fichiers] IS NOT NULL OR [Fk_Foxtable] IS NOT NULL AND [Fk_Fichiers] IS NULL))
		END 
		;
		
	/****** Object:  Schema [historique]    Script Date: 16/08/2017 14:37:43 ******/
	IF NOT EXISTS (SELECT * FROM sys.schemas WHERE name = N'historique')
	EXEC sys.sp_executesql N'CREATE SCHEMA [historique] AUTHORIZATION [db_securityadmin]'
	;

	/****** Object:  Table [historique].[ddl_log]    Script Date: 16/08/2017 14:37:43 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[historique].[ddl_log]') AND type in (N'U'))
	BEGIN
	CREATE TABLE [historique].[ddl_log](
		[PostTime] [datetime] NULL,
		[DB_User] [nvarchar](100) NULL,
		[Event] [nvarchar](100) NULL,
		[TSQL] [nvarchar](2000) NULL
	) ON [PRIMARY]
	END
	;

	/****** Object:  DdlTrigger [table_alter_drop_securite]    Script Date: 16/08/2017 14:37:43 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.triggers WHERE parent_class_desc = 'DATABASE' AND name = N'table_alter_drop_securite')
	EXECUTE dbo.sp_executesql N'
	CREATE TRIGGER [table_alter_drop_securite] ON DATABASE
	FOR DROP_TABLE, ALTER_TABLE
	AS
	     DECLARE @Data XML;
	     SET @Data = EVENTDATA();

	     DECLARE @Evenement NVARCHAR(100),
	             @Nomtable  NVARCHAR(100),
	             @TSQL      NVARCHAR(2000),
			     @Login	    NVARCHAR(100),
			     @origine   NVARCHAR(100);

	     SET @Evenement	= @Data.value(''(/EVENT_INSTANCE/EventType)[1]'', ''nvarchar(100)'');
	     SET @Nomtable	= @Data.value(''(/EVENT_INSTANCE/ObjectName)[1]'', ''nvarchar(100)'');
	     SET @TSQL		= @Data.value(''(/EVENT_INSTANCE/TSQLCommand)[1]'', ''nvarchar(2000)'');
		 SET @Login	    = @Data.value(''(/EVENT_INSTANCE/LoginName)[1]'', ''nvarchar(100)'');
		 SET @origine	= ORIGINAL_LOGIN()

	     IF @Nomtable IN (''foxtable'',''fichiers'')
	         BEGIN

	             INSERT INTO Historique.Ddl_Log
	                  (Posttime,
	                   Db_User,
	                   Event,
	                   TSQL
	                  )
	             VALUES
	                  (GETDATE(),
	                   @origine + '' || ''+   @Login + '' || '' + CONVERT(NVARCHAR(100), CURRENT_USER)  ,
	                   @Evenement,
	                   @TSQL
	                  );
	     END;


	    IF @Nomtable = ''ddl_log''
		   BEGIN
			  PRINT ''Interdit de supprimer ou modifier cette table'';
			  ROLLBACK TRANSACTION;
	    END;

	;
	ENABLE TRIGGER [table_alter_drop_securite] ON DATABASE
	;
	GRANT EXECUTE, SELECT, INSERT, UPDATE, DELETE ON SCHEMA ::dbo TO public;
	DENY DELETE, UPDATE ON SCHEMA ::historique TO public;
	'
		ENDTEXT

		SQLEXEC(m.lnCnxHandle, m.lcTablesCreate ,"curresult")

		* on vrifie que le dossier partag est accessible, et si oui, on le met dans une variable dclare private en haut
		* il ne faut pas vrifier si le dossier est accessible, les droits d'accs  la machine distante ne sont peut-tre
		* pas encore dfinis
		* select filetablerootpath() as chemin
		SQLEXEC(m.lnCnxHandle, "USE [" + ALLTRIM(m.dbName)+"] ; SELECT CAST(FileTableRootPath() AS varchar(254)) AS chemin" ,"curresult")
		LOCAL lcChemin AS STRING
		m.lcChemin=ALLTRIM(curresult.chemin)
		*		IF DIRECTORY(ADDBS(m.lcChemin)+"depotfox")
		STORE ADDBS(m.lcChemin)+"depotfox" TO m.gcVirtualPath
		m.llResult=.T.
		*		ELSE
		*			m.llResult=.F.
		*		ENDIF

	CATCH TO loErr
		=AERROR(aErraray)

		MESSAGEBOX("plopTables")

	FINALLY

		USE IN SELECT("curresult")

	ENDTRY

	WAIT CLEAR
	RETURN m.llResult
ENDFUNC && CreateTables

***********************************************************
FUNCTION CreateIndexes AS Boolean
	LPARAMETERS dbName

	LOCAL llResult AS Boolean

	LOCAL lcDbArchiveName AS STRING
	m.lcDbArchiveName=ALLTRIM(m.dbName)+"_archive"

	TRY
		TEXT TO lcIndexesCreate NOSHOW TEXTMERGE
		USE [<<ALLTRIM(m.dbName)>>] ;
		SET ANSI_PADDING ON

		;		
		/****** Object:  Index [IX_TYP_nom]    Script Date: 03/12/2017 15:44:41 ******/
		IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TYPE_FICHIERS]') AND name = N'IX_TYP_nom')
		CREATE NONCLUSTERED INDEX [IX_TYP_nom] ON [dbo].[TYPE_FICHIERS]
		(
			[TYP_nom] ASC
		)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
		;
		IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DF_TYPE_FICHIERS_TYP_actif]') AND type = 'D')
		BEGIN
		ALTER TABLE [dbo].[TYPE_FICHIERS] ADD  CONSTRAINT [DF_TYPE_FICHIERS_TYP_actif]  DEFAULT ((0)) FOR [TYP_actif]
		END

		;
		
		USE [<<m.lcDbArchiveName>>] ;
		SET ANSI_PADDING ON

		;
		/****** Object:  Index [IX_foxtable_nomtable]    Script Date: 19/07/2017 12:03:29 ******/
		IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[foxtable]') AND name = N'IX_foxtable_nomtable')
		CREATE NONCLUSTERED INDEX [IX_foxtable_nomtable] ON [dbo].[foxtable]
		(
			[fox_nom_table] ASC
		)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
		;
		SET ANSI_PADDING ON

		;
		/****** Object:  Index [IX_Projet_nom]    Script Date: 19/07/2017 12:03:29 ******/
		IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Projet]') AND name = N'IX_Projet_nom')
		CREATE NONCLUSTERED INDEX [IX_Projet_nom] ON [dbo].[Projet]
		(
			[nom_projet] ASC
		)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
		;
		/****** Object:  Index [IX_Projet_niveau]    Script Date: 28/08/2017 15:17:25 ******/
		IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Projet]') AND name = N'IX_Projet_niveau')
		CREATE NONCLUSTERED INDEX [IX_Projet_niveau] ON [dbo].[Projet]
		(
			[niveau] ASC
		)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
		;
		SET ANSI_PADDING ON

		;
		/****** Object:  Index [IX_Version_FKfoxtable]    Script Date: 24/09/2017 18:02:53 ******/
		IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Version]') AND name = N'IX_Version_FKfoxtable')
		CREATE NONCLUSTERED INDEX [IX_Version_FKfoxtable] ON [dbo].[Version]
		(
			[FK_foxtable] ASC
		)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
		;
		SET ANSI_PADDING ON
		;
		/****** Object:  Index [IX_Version_FKfichiers]    Script Date: 12/12/2017 19:31:19 ******/
		CREATE NONCLUSTERED INDEX [IX_Version_FKfichiers] ON [dbo].[Version]
		(
			[FK_fichiers] ASC
		)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
		;		
		SET ANSI_PADDING ON	
		;

		/****** Object:  Index [UQ_Version_FKfichiers_debut]    Script Date: 19/12/2017 17:10:13 ******/
		IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Version]') AND name = N'UQ_Version_FKfichiers_debut')
		CREATE UNIQUE NONCLUSTERED INDEX [UQ_Version_FKfichiers_debut] ON [dbo].[Version]
		(
			[FK_fichiers] ASC,
			[debut] ASC
		)
		WHERE ([FK_foxtable] IS NOT NULL)
		WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
		;
		SET ANSI_PADDING ON	
		;		
		/****** Object:  Index [UQ_Version_FKfichiers_numero]    Script Date: 19/12/2017 17:10:13 ******/
		IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Version]') AND name = N'UQ_Version_FKfichiers_numero')
		CREATE UNIQUE NONCLUSTERED INDEX [UQ_Version_FKfichiers_numero] ON [dbo].[Version]
		(
			[numeroID] ASC,
			[FK_fichiers] ASC
		)
		WHERE ([FK_fichiers] IS NOT NULL)
		WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
		;
		SET ANSI_PADDING ON	
		;		
		/****** Object:  Index [UQ_Version_FKfoxtable_debut]    Script Date: 19/12/2017 17:10:13 ******/
		IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Version]') AND name = N'UQ_Version_FKfoxtable_debut')
		CREATE UNIQUE NONCLUSTERED INDEX [UQ_Version_FKfoxtable_debut] ON [dbo].[Version]
		(
			[FK_foxtable] ASC,
			[debut] ASC
		)
		WHERE ([FK_foxtable] IS NOT NULL)
		WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
		;
		SET ANSI_PADDING ON	
		;		
		/****** Object:  Index [UQ_Version_FKfoxtable_numero]    Script Date: 19/12/2017 17:10:13 ******/
		IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Version]') AND name = N'UQ_Version_FKfoxtable_numero')
		CREATE UNIQUE NONCLUSTERED INDEX [UQ_Version_FKfoxtable_numero] ON [dbo].[Version]
		(
			[numeroID] ASC,
			[FK_foxtable] ASC
		)
		WHERE ([FK_foxtable] IS NOT NULL)
		WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
		;

		/****** Object:  Index [UQ_FoxTable_Fichiers_Projet_FKFichiers_FK_Projet]    Script Date: 18/01/2018 10:47:14 ******/
		IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[FoxTable_Fichiers_Projet]') AND name = N'UQ_FoxTable_Fichiers_Projet_FKFichiers_FK_Projet')
		CREATE UNIQUE NONCLUSTERED INDEX [UQ_FoxTable_Fichiers_Projet_FKFichiers_FK_Projet] ON [dbo].[FoxTable_Fichiers_Projet]
		(
			[FK_Projet] ASC,
			[FK_Fichiers] ASC
		)
		WHERE ([FK_fichiers] IS NOT NULL)
		WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
		;
		/****** Object:  Index [UQ_FoxTable_Fichiers_Projet_FKfoxtable_FK_Projet]    Script Date: 18/01/2018 10:47:14 ******/
		IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[FoxTable_Fichiers_Projet]') AND name = N'UQ_FoxTable_Fichiers_Projet_FKfoxtable_FK_Projet')
		CREATE UNIQUE NONCLUSTERED INDEX [UQ_FoxTable_Fichiers_Projet_FKfoxtable_FK_Projet] ON [dbo].[FoxTable_Fichiers_Projet]
		(
			[FK_Projet] ASC,
			[FK_FoxTable] ASC
		)
		WHERE ([FK_foxtable] IS NOT NULL)
		WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
		;


		ENDTEXT

		SQLEXEC(m.lnCnxHandle, m.lcindexesCreate ,"curresult")

		* vrification de la cration des index
		TEXT TO lcSqlVerif  NOSHOW TEXTMERGE
		SELECT COUNT(*) AS Nb
		     FROM
		          Sys.Indexes
		     WHERE Name IN(
			N'IX_foxtable_nomtable', 
			N'IX_Projet_nom',
			N'IX_Projet_niveau', 
			N'IX_Version_FKfoxtable', 
			N'IX_Version_FKfichiers',
			N'UQ_Version_FKfichiers_debut',
			N'UQ_Version_FKfichiers_numero',
			N'UQ_Version_FKfoxtable_debut',
			N'UQ_Version_FKfoxtable_numero',
			N'UQ_FoxTable_Fichiers_Projet_FKFichiers_FK_Projet',
			N'UQ_FoxTable_Fichiers_Projet_FKfoxtable_FK_Projet');
		ENDTEXT
		SQLEXEC(m.lnCnxHandle, m.lcSqlVerif ,"curresult")

		IF curresult.nb=11
			m.llResult = .T.
		ELSE
			m.llResult = .F.
		ENDIF


	CATCH
		=AERROR(aErraray)
		MESSAGEBOX("plopIndexes")

	FINALLY
		USE IN SELECT("curresult")
	ENDTRY


	WAIT CLEAR
	RETURN m.llResult
ENDFUNC && CreateIndexes

******************************************************
FUNCTION CreateFK
	LPARAMETERS dbName

	LOCAL llResult AS Boolean

	LOCAL lcDbArchiveName AS STRING
	m.lcDbArchiveName=ALLTRIM(m.dbName)+"_archive"
	TRY
		TEXT TO lcFKcreate NOSHOW TEXTMERGE
	USE [<<m.lcDbArchiveName>>] ;

	IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FoxTable_Fichiers_Projet_fichiers]') AND parent_object_id = OBJECT_ID(N'[dbo].[FoxTable_Fichiers_Projet]'))
	ALTER TABLE [dbo].[FoxTable_Fichiers_Projet]  WITH CHECK ADD  CONSTRAINT [FK_FoxTable_Fichiers_Projet_fichiers] FOREIGN KEY([FK_Fichiers])
	REFERENCES [dbo].[fichiers] ([PK])
	ON DELETE SET NULL
	;
	IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FoxTable_Fichiers_Projet_fichiers]') AND parent_object_id = OBJECT_ID(N'[dbo].[FoxTable_Fichiers_Projet]'))
	ALTER TABLE [dbo].[FoxTable_Fichiers_Projet] CHECK CONSTRAINT [FK_FoxTable_Fichiers_Projet_fichiers]
	;
	IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FoxTable_Fichiers_Projet_foxtable]') AND parent_object_id = OBJECT_ID(N'[dbo].[FoxTable_Fichiers_Projet]'))
	ALTER TABLE [dbo].[FoxTable_Fichiers_Projet]  WITH CHECK ADD  CONSTRAINT [FK_FoxTable_Fichiers_Projet_foxtable] FOREIGN KEY([FK_FoxTable])
	REFERENCES [dbo].[foxtable] ([PK])
	ON DELETE SET NULL
	;
	IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FoxTable_Fichiers_Projet_foxtable]') AND parent_object_id = OBJECT_ID(N'[dbo].[FoxTable_Fichiers_Projet]'))
	ALTER TABLE [dbo].[FoxTable_Fichiers_Projet] CHECK CONSTRAINT [FK_FoxTable_Fichiers_Projet_foxtable]
	;
	IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FoxTable_Fichiers_Projet_Projet]') AND parent_object_id = OBJECT_ID(N'[dbo].[FoxTable_Fichiers_Projet]'))
	ALTER TABLE [dbo].[FoxTable_Fichiers_Projet]  WITH CHECK ADD  CONSTRAINT [FK_FoxTable_Fichiers_Projet_Projet] FOREIGN KEY([FK_Projet])
	REFERENCES [dbo].[Projet] ([PK])
	;
	IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_FoxTable_Fichiers_Projet_Projet]') AND parent_object_id = OBJECT_ID(N'[dbo].[FoxTable_Fichiers_Projet]'))
	ALTER TABLE [dbo].[FoxTable_Fichiers_Projet] CHECK CONSTRAINT [FK_FoxTable_Fichiers_Projet_Projet]
	;
	IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Version_Fichiers]') AND parent_object_id = OBJECT_ID(N'[dbo].[Version]'))
	ALTER TABLE [dbo].[Version]  WITH CHECK ADD  CONSTRAINT [FK_Version_Fichiers] FOREIGN KEY([FK_fichiers])
	REFERENCES [dbo].[fichiers] ([PK])
	;
	IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Version_Fichiers]') AND parent_object_id = OBJECT_ID(N'[dbo].[Version]'))
	ALTER TABLE [dbo].[Version] CHECK CONSTRAINT [FK_Version_Fichiers]
	;
	IF NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Version_FoxTable]') AND parent_object_id = OBJECT_ID(N'[dbo].[Version]'))
	ALTER TABLE [dbo].[Version]  WITH CHECK ADD  CONSTRAINT [FK_Version_FoxTable] FOREIGN KEY([FK_foxtable])
	REFERENCES [dbo].[foxtable] ([PK])
	;
	IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Version_FoxTable]') AND parent_object_id = OBJECT_ID(N'[dbo].[Version]'))
	ALTER TABLE [dbo].[Version] CHECK CONSTRAINT [FK_Version_FoxTable]
	;
		ENDTEXT

		SQLEXEC(m.lnCnxHandle, m.lcFKCreate ,"curresult")

		* vrification
		TEXT TO lcSqlVerif  NOSHOW TEXTMERGE
		select COUNT(*) AS nb FROM sys.foreign_keys WHERE parent_object_id IN (OBJECT_ID(N'[dbo].[FoxTable_Fichiers_Projet]'), OBJECT_ID(N'[dbo].[Version]'))
		ENDTEXT
		SQLEXEC(m.lnCnxHandle, m.lcSqlVerif ,"curresult")

		IF curresult.nb=5
			m.llResult = .T.
		ELSE
			m.llResult = .F.
		ENDIF

	CATCH
		=AERROR(aErraray)
		MESSAGEBOX("plopFK")
	FINALLY
		USE IN SELECT("curresult")
	ENDTRY


	WAIT CLEAR
	RETURN m.llResult
ENDFUNC && CreateFK

*************************************************
FUNCTION CreateProc AS Boolean
	LPARAMETERS dbName

	LOCAL llResult AS Boolean

	LOCAL lcDbArchiveName AS STRING
	m.lcDbArchiveName=ALLTRIM(m.dbName)+"_archive"
	* il faut imprativement que le CREATE PROC soit la 1re instruction, donc chaque SP doit tre cre dans un batch spcifique

	TRY
		** on revient dans le contexte de dbname
		SQLEXEC(m.lnCnxHandle, "USE [" + ALLTRIM(m.dbName)+"] ;")
		** la procedure Merge
		TEXT TO lcProceduresCreate NOSHOW TEXTMERGE

	/****** Object:  StoredProcedure [dbo].[Merge_FoxTable_depuis_DepotPourFox]    Script Date: 08/11/2017 16:38:40 ******/

	-- =============================================
	-- Author:		Michel
	-- Create date: 2017-11-08
	-- Description:	Insertion ou Modification dans FoxTable depuis le contenu de DepotPourFox
	--              ajout du traitement des prg, mpr, qpr
	--              correction du bug sur noms de fichiers comprenant des .
	-- =============================================

	CREATE PROCEDURE [dbo].[Merge_FoxTable_depuis_DepotPourFox]

	AS
	BEGIN
		-- SET NOCOUNT ON added to prevent extra result sets from
		-- interfering with SELECT statements.
		SET NOCOUNT ON;

	-- extraction des donnes ncessaires
	;
	WITH
	CTEprepare
	     AS (
	     SELECT -- Attention, la fonction GetFileNamespacePath est sensible  la casse, ne pas modifier son nom!!
	            REVERSE( SUBSTRING( REVERSE( File_Stream.GetFileNamespacePath( ) ), CHARINDEX( '\', REVERSE( File_Stream.GetFileNamespacePath( ) ), 1 ), LEN( File_Stream.GetFileNamespacePath( ) ) ) ) AS Chemin,
	            REVERSE(LEFT(REVERSE([Name]),CHARINDEX('.',REVERSE([Name]),0)-1  )) AS Fox_Type_Table,
	            REVERSE(SUBSTRING(REVERSE([Name]),CHARINDEX('.',REVERSE([Name]),0)+1,255)) AS Fox_Nom_Table,
	            File_Stream,
	            File_Type
	          FROM
	               Dbo.Depotpourfox
	          WHERE Is_Directory = 0),

	-- les fichiers de donnes
	CTEdbf
	     AS (
	     SELECT Chemin,
	            Fox_Type_Table,
	            Fox_Nom_Table,
	            [Dbf] AS "Fox_Data",
	            [Cdx] AS "Fox_Index",
	            [Fpt] AS "Fox_Memo"
	          FROM
	               CTEprepare PIVOT(MAX(File_Stream) FOR File_Type IN([Dbf],
	                                                                    [Cdx],
	                                                                    [Fpt])) AS Pvt),
	-- les fichiers de database
	 CTEdbc
	     AS (
	     SELECT Chemin,
			  Fox_Type_Table,
	            Fox_Nom_Table,
	            [Dbc] AS "Fox_Data",
	            [dcx] AS "Fox_Index",
	            [dct] AS "Fox_Memo"
	          FROM
	               CTEprepare PIVOT(MAX(File_Stream) FOR File_Type IN([Dbc],
	                                                                    [dcx],
	                                                                    [dct])) AS Pvt),
	-- les fichiers de projet
	     CTEproj
	     AS (
	     SELECT Chemin,
			  Fox_Type_Table,
	            Fox_Nom_Table,
	            [pjx] AS "Fox_Data",
	            NULL AS "Fox_Index",
	            [pjt] AS "Fox_Memo"
	          FROM
	               CTEprepare PIVOT(MAX(File_Stream) FOR File_Type IN([pjx],
	                                                                    [pjt])) AS Pvt),
	-- les fichiers de formulaire
	     CTEform
	     AS (
	     SELECT Chemin,
			  Fox_Type_Table,
	            Fox_Nom_Table,
	            [Scx] AS "Fox_Data",
	            NULL AS "Fox_Index",
	            [Sct] AS "Fox_Memo"
	          FROM
	               CTEprepare PIVOT(MAX(File_Stream) FOR File_Type IN([Scx],
	                                                                    [Sct])) AS Pvt),

	-- les fichiers de classe
	     CTEclasse
	     AS (
	     SELECT Chemin,
			  Fox_Type_Table,
	            Fox_Nom_Table,
	            [vcx] AS "Fox_Data",
	            NULL AS "Fox_Index",
	            [vct] AS "Fox_Memo"
	          FROM
	               CTEprepare PIVOT(MAX(File_Stream) FOR File_Type IN([vcx],
	                                                                    [vct])) AS Pvt),

	-- les fichiers de rapport
	     CTErapport
	     AS (
	     SELECT Chemin,
			  Fox_Type_Table,
	            Fox_Nom_Table,
	            [frx] AS "Fox_Data",
	            NULL AS "Fox_Index",
	            [frt] AS "Fox_Memo"
	          FROM
	               CTEprepare PIVOT(MAX(File_Stream) FOR File_Type IN([frx],
	                                                                    [frt])) AS Pvt),

	-- les fichiers d'tiquette
	     CTEetiquette
	     AS (
	     SELECT Chemin,
			  Fox_Type_Table,
	            Fox_Nom_Table,
	            [lbx] AS "Fox_Data",
	            NULL AS "Fox_Index",
	            [lbt] AS "Fox_Memo"
	          FROM
	               CTEprepare PIVOT(MAX(File_Stream) FOR File_Type IN([lbx],
	                                                                    [lbt])) AS Pvt),

	-- les fichiers de menu
	     CTEmenu
	     AS (
	     SELECT Chemin,
			  Fox_Type_Table,
	            Fox_Nom_Table,
	            [mnx] AS "Fox_Data",
	            NULL AS "Fox_Index",
	            [mnt] AS "Fox_Memo"
	          FROM
	               CTEprepare PIVOT(MAX(File_Stream) FOR File_Type IN([mnx],
	                                                                    [mnt])) AS Pvt),

    -- les fichiers de programme
			CTEprg as (
			SELECT Chemin,
	            Fox_Type_Table,
	            Fox_Nom_Table,
	            file_stream AS "Fox_Data",
	            NULL AS "Fox_Index",
	            NULL AS "Fox_Memo"
	          FROM
	               CTEprepare where file_type='prg'),

    -- les fichiers de prog menus
			CTEmpr as (
			SELECT Chemin,
	            Fox_Type_Table,
	            Fox_Nom_Table,
	            file_stream AS "Fox_Data",
	            NULL AS "Fox_Index",
	            NULL AS "Fox_Memo"
	          FROM
	               CTEprepare where file_type='mpr'),

    -- les fichiers plats de requte
			CTEqpr as (
			SELECT Chemin,
	            Fox_Type_Table,
	            Fox_Nom_Table,
	            file_stream AS "Fox_Data",
	            NULL AS "Fox_Index",
	            NULL AS "Fox_Memo"
	          FROM
	               CTEprepare where file_type='qpr'),


	-- on assemble les CTE
	CTEfinale
	AS(
	-- donnes
	     SELECT 'donnes' as fox_typetable,
			  chemin,
			  Fox_Nom_Table,
	            MAX(Fox_Data) AS Fox_Data,
	            MAX(Fox_Index) AS Fox_Index,
	            MAX(Fox_Memo) AS Fox_Memo
	          FROM
	               CTEdbf
	          WHERE Fox_Type_Table IN('dbf', 'cdx', 'fpt')
	          GROUP BY Chemin, Fox_Nom_Table

	     UNION ALL

	-- database
	    SELECT 'database' as fox_typetable,
			  chemin,
			  Fox_Nom_Table,
	            MAX(Fox_Data) AS Fox_Data,
	            MAX(Fox_Index) AS Fox_Index,
	            MAX(Fox_Memo) AS Fox_Memo
	          FROM
	               CTEdbc
	          WHERE Fox_Type_Table IN('dbc', 'dcx', 'dct')
	          GROUP BY Chemin, Fox_Nom_Table

	     UNION ALL

	-- projet
	     SELECT 'projet' as fox_typetable,
			  chemin,
			  Fox_Nom_Table,
	            MAX(Fox_Data) AS Fox_Data,
	            NULL AS Fox_Index,
	            MAX(Fox_Memo) AS Fox_Memo
	          FROM
	               CTEproj
	          WHERE Fox_Type_Table IN('pjx', 'pjt')
	          GROUP BY Chemin, Fox_Nom_Table

		UNION ALL

	-- form
	     SELECT 'form' as fox_typetable,
			  chemin,
			  Fox_Nom_Table,
	            MAX(Fox_Data) AS Fox_Data,
	            NULL AS Fox_Index,
	            MAX(Fox_Memo) AS Fox_Memo
	          FROM
	               CTEform
	          WHERE Fox_Type_Table IN('scx', 'sct')
	          GROUP BY Chemin, Fox_Nom_Table

	    UNION ALL

	-- classe
	     SELECT 'classe' as fox_typetable,
			  chemin,
			  Fox_Nom_Table,
	            MAX(Fox_Data) AS Fox_Data,
	            NULL AS Fox_Index,
	            MAX(Fox_Memo) AS Fox_Memo
	          FROM
	               CTEclasse
	          WHERE Fox_Type_Table IN('vcx', 'vct')
	          GROUP BY chemin, Fox_Nom_Table

	    UNION ALL

	-- rapport
	     SELECT 'rapport' as fox_typetable,
			  chemin,
			  Fox_Nom_Table,
	            MAX(Fox_Data) AS Fox_Data,
	            NULL AS Fox_Index,
	            MAX(Fox_Memo) AS Fox_Memo
	          FROM
	               CTErapport
	          WHERE Fox_Type_Table IN('frx', 'frt')
	          GROUP BY Chemin, Fox_Nom_Table

	    UNION ALL

	-- etiquette
	     SELECT 'tiquette' as fox_typetable,
			  chemin,
			  Fox_Nom_Table,
	            MAX(Fox_Data) AS Fox_Data,
	            NULL AS Fox_Index,
	            MAX(Fox_Memo) AS Fox_Memo
	          FROM
	               CTEetiquette
	          WHERE Fox_Type_Table IN('lbx', 'lbt')
	          GROUP BY Chemin, Fox_Nom_Table

	    UNION ALL

	-- menu
	     SELECT 'menu' as fox_typetable,
			  chemin,
			  Fox_Nom_Table,
	            MAX(Fox_Data) AS Fox_Data,
	            NULL AS Fox_Index,
	            MAX(Fox_Memo) AS Fox_Memo
	          FROM
	               CTEmenu
	          WHERE Fox_Type_Table IN('mnx', 'mnt')
	          GROUP BY Chemin, Fox_Nom_Table

         UNION ALL

    -- programmes
	   SELECT 'programmes' as fox_typetable,
			  chemin,
			  Fox_Nom_Table,
	            MAX(Fox_Data) AS Fox_Data,
	            NULL AS Fox_Index,
	            NULL AS Fox_Memo
	          FROM
	               CTEprg
	          WHERE Fox_Type_Table = 'prg'
	          GROUP BY Chemin, Fox_Nom_Table

         UNION ALL

    -- menus  plat
	   SELECT 'menusprog' as fox_typetable,
			  chemin,
			  Fox_Nom_Table,
	            MAX(Fox_Data) AS Fox_Data,
	            NULL AS Fox_Index,
	            NULL AS Fox_Memo
	          FROM
	               CTEmpr
	          WHERE Fox_Type_Table = 'mpr'
	          GROUP BY Chemin, Fox_Nom_Table

         UNION ALL

    -- requtes  plat
	   SELECT 'queryprog' as fox_typetable,
			  chemin,
			  Fox_Nom_Table,
	            MAX(Fox_Data) AS Fox_Data,
	            NULL AS Fox_Index,
	            NULL AS Fox_Memo
	          FROM
	               CTEqpr
	          WHERE Fox_Type_Table = 'qpr'
	          GROUP BY Chemin, Fox_Nom_Table
	)

	-- on envoie les donnes rsultantes vers la table d'historisation

	MERGE [<<m.lcDbArchiveName>>].dbo.FoxTable AS T
	USING CTEfinale AS S
	ON( T.Fox_nom_table = S.Fox_nom_table AND
		T.fox_chemin = S.chemin
	  )
	WHEN MATCHED AND
		   (	  (	 T.Fox_data <> S.Fox_data
			   OR (T.Fox_data IS NOT NULL AND S.Fox_data IS NULL)
			   OR (T.Fox_data IS NULL AND S.Fox_data IS NOT NULL))
		   OR  (	 T.Fox_index <> S.Fox_index
			   OR (T.Fox_index IS NOT NULL AND S.Fox_index IS NULL)
			   OR (T.Fox_index IS NULL AND S.Fox_index IS NOT NULL))
		   OR  (	 T.Fox_memo <> S.Fox_memo
			   OR (T.Fox_memo IS NOT NULL AND S.Fox_memo IS NULL)
			   OR (T.Fox_memo IS NULL AND S.Fox_memo IS NOT NULL)))
		  THEN UPDATE SET T.fox_data = S.fox_data,
					   T.fox_index = S.fox_index,
					   T.fox_memo = S.fox_memo
	WHEN NOT MATCHED BY TARGET
		  THEN
		   INSERT (Fox_Chemin,
				 Fox_Typetable,
				 Fox_Nom_Table,
				 Fox_Data,
				 Fox_Index,
				 Fox_Memo)
	      VALUES
				  (S.Chemin,
				   S.Fox_Typetable,
				   S.Fox_Nom_Table,
				   S.Fox_Data,
				   S.Fox_Index,
				   S.Fox_Memo
				  )
		OUTPUT $action ;

	--OUTPUT $action, Inserted.*, Deleted.*;

	END

		ENDTEXT

		* on vrifie la non-existence avant de lancer l'exec
		TEXT TO lcSqlVerif  NOSHOW TEXTMERGE
		SELECT COUNT(*) as nb FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Merge_FoxTable_depuis_DepotPourFox]') AND type in (N'P', N'PC')
		ENDTEXT
		SQLEXEC(m.lnCnxHandle, m.lcSqlVerif ,"curresult")
		IF curresult.nb=0
			SQLEXEC(m.lnCnxHandle, m.lcProceduresCreate ,"curresult")
		ENDIF

		* la procedure Merge_Fichiers_depuis_DepotPourFox
	TEXT TO lcProceduresCreate  NOSHOW TEXTMERGE

	-- =============================================
	-- Author:		Michel
	-- Create date: 2017-12-01
	-- Description:	Insertion ou Modification dans Fichiers depuis le contenu de DepotPourFox
	-- =============================================
	CREATE PROCEDURE [dbo].[Merge_Fichiers_depuis_DepotPourFox] 

	AS
	BEGIN
		-- SET NOCOUNT ON added to prevent extra result sets from
		-- interfering with SELECT statements.
		SET NOCOUNT ON;
		-- extraction des donnes ncessaires
	WITH CTEprepare
	     AS (
	     SELECT -- Attention, la fonction GetFileNamespacePath est sensible  la casse, ne pas modifier son nom!!
	     Reverse(SUBSTRING(REVERSE(File_Stream.GetFileNamespacePath()), CHARINDEX('\', REVERSE(File_Stream.GetFileNamespacePath()), 1), LEN(File_Stream.GetFileNamespacePath()))) AS Chemin,
	     [Name] AS Nom_Complet,
	     REVERSE(SUBSTRING(REVERSE([Name]), CHARINDEX('.', REVERSE([Name]), 0)+1, 255)) AS Nom_Fichier,
	     File_Stream AS Contenu_Fichier,
	     File_Type
	          FROM
	               Dbo.Depotpourfox
	          WHERE Is_Directory = 0)

	,

	CTEsignaturespossibles
	     AS (
	     SELECT -- Attention, la fonction GetFileNamespacePath est sensible  la casse, ne pas modifier son nom!!
	     Reverse(SUBSTRING(REVERSE(File_Stream.GetFileNamespacePath()), CHARINDEX('\', REVERSE(File_Stream.GetFileNamespacePath()), 1), LEN(File_Stream.GetFileNamespacePath()))) AS Chemin,
	     [Name] AS Nom_Complet_Signature,
	     REVERSE(SUBSTRING(REVERSE([Name]), CHARINDEX('.', REVERSE([Name]), 0)+1, 255)) AS Nom_Fichier,
	     File_Stream AS Contenu_Signature,
	     File_Type,
	     IIF(PATINDEX('%<xades:QualifyingProperties%', COALESCE(CAST(CAST(File_Stream AS XML) AS NVARCHAR(MAX)), N'')) > 0
	         AND PATINDEX('%/xades:QualifyingProperties>%', COALESCE(CAST(CAST(File_Stream AS XML) AS NVARCHAR(MAX)), N'')) > 0, 1, 0) AS Oksign
	          FROM
	               Dbo.Depotpourfox
	          WHERE Is_Directory = 0
	                AND File_Type = 'xml')
	,

	CTEfinale
	    AS (
	         SELECT P.Chemin AS fichier_chemin,	
	            P.Nom_Complet AS fichier_nom,			
	            P.Contenu_Fichier AS fichier_data,		
			  T.TYP_nom AS fichier_type ,				
	            S.Nom_Complet_Signature AS signature_nom ,	      
	            S.Contenu_Signature AS signature_data ,		     
	            S.Oksign
	          FROM
	               Cteprepare P
	               JOIN Dbo.Type_Fichiers T
	                       ON P.File_Type = T.Typ_Ext
	                          AND T.Typ_Actif = 1
	               LEFT OUTER JOIN Ctesignaturespossibles S
	                       ON P.Nom_Complet = S.Nom_Fichier
						 AND P.Chemin  =S.Chemin
	)

		-- on envoie les donnes rsultantes vers la table d'historisation
		-- les colonnes qui peuvent etre updates sont fichier_data, fichier_type, signature_nom, signature_data

		   MERGE [<<m.lcDbArchiveName>>].Dbo.Fichiers AS Trg
		   USING Ctefinale AS Src
		   ON(Trg.Fichier_Nom = Src.Fichier_Nom
			 AND Trg.Fichier_Chemin = Src.Fichier_Chemin)
			  WHEN MATCHED AND -- ici les conditions sur NULL dans les colonnes  updater
		   ((Trg.Fichier_Data <> Src.Fichier_Data
			OR (Trg.Fichier_Data IS NULL
			    AND Src.Fichier_Data IS NOT NULL)
			OR (Trg.Fichier_Data IS NOT NULL
			    AND Src.Fichier_Data IS NULL))
		    OR (Trg.Fichier_Type <> Src.Fichier_Type
			   OR (Trg.Fichier_Type IS NULL
				  AND Src.Fichier_Type IS NOT NULL)
			   OR (Trg.Fichier_Type IS NOT NULL
				  AND Src.Fichier_Type IS NULL))
		    OR (Trg.Signature_Nom <> Src.Signature_Nom
			   OR (Trg.Signature_Nom IS NULL
				  AND Src.Signature_Nom IS NOT NULL)
			   OR (Trg.Signature_Nom IS NOT NULL
				  AND Src.Signature_Nom IS NULL))
		    OR (Trg.Signature_Data <> Src.Signature_Data
			   OR (Trg.Signature_Data IS NULL
				  AND Src.Signature_Data IS NOT NULL)
			   OR (Trg.Signature_Data IS NOT NULL
				  AND Src.Signature_Data IS NULL)))
				 THEN UPDATE SET
							  Trg.Fichier_Data = Src.Fichier_Data,
							  Trg.Fichier_Type = Src.Fichier_Type,
							  Trg.Signature_Nom = Src.Signature_Nom,
							  Trg.Signature_Data = Src.Signature_Data
			  WHEN NOT MATCHED BY TARGET
				 THEN
			    INSERT(Fichier_Nom,
					 Fichier_Chemin,
					 Fichier_Data,
					 Fichier_Type,
					 Signature_Nom,
					 Signature_Data)
			    VALUES
					 (Src.Fichier_Nom,
					  Src.Fichier_Chemin,
					  Src.Fichier_Data,
					  Src.Fichier_Type,
					  CASE Src.Oksign
						 WHEN 1
							THEN Src.Signature_Nom
					  END,
					  CASE Src.Oksign
						 WHEN 1
							THEN Src.Signature_Data
					  END
			   )
			   OUTPUT $ACTION ;

	END

	ENDTEXT		
	
		* on vrifie la non-existence avant de lancer l'exec
		TEXT TO lcSqlVerif  NOSHOW TEXTMERGE
		SELECT COUNT(*) as nb FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Merge_Fichiers_depuis_DepotPourFox]') AND type in (N'P', N'PC')
		ENDTEXT
		SQLEXEC(m.lnCnxHandle, m.lcSqlVerif ,"curresult")
		IF curresult.nb=0
			SQLEXEC(m.lnCnxHandle, m.lcProceduresCreate ,"curresult")
		ENDIF

		* la procdure ZAP historique foxtable
		** on revient dans le contexte de dbname_archive
		SQLEXEC(m.lnCnxHandle, "USE [" + ALLTRIM(m.lcDbArchiveName)+"] ;")
		TEXT TO lcProceduresCreate NOSHOW TEXTMERGE

	/****** Object:  StoredProcedure [dbo].[Zap_Historique_FoxTable]    Script Date: 19/07/2017 12:03:29 ******/

	-- =============================================
	-- Author:		Michel
	-- Create date: 2017-07-18
	-- Description:	Purge en totalit l'historique
	-- =============================================
	CREATE PROCEDURE [dbo].[Zap_Historique_FoxTable]
	WITH EXECUTE AS SELF
	AS
	BEGIN
		-- SET NOCOUNT ON added to prevent extra result sets from
		-- interfering with SELECT statements.
		SET NOCOUNT ON;

	BEGIN TRAN;

	    ALTER TABLE Dbo.Foxtable SET(SYSTEM_VERSIONING = OFF);
	    EXEC (' delete from dbo.foxtable_historique');
	    ALTER TABLE Dbo.Foxtable SET(SYSTEM_VERSIONING = ON(HISTORY_TABLE = Dbo.Foxtable_Historique));
	    DELETE FROM dbo.FoxTable_Fichiers_Projet WHERE FK_FoxTable IS NULL;

	COMMIT;

	END
		ENDTEXT

		* on vrifie la non-existence avant de lancer l'exec
		TEXT TO lcSqlVerif  NOSHOW TEXTMERGE
		SELECT COUNT(*) as nb FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Zap_Historique_FoxTable]') AND type in (N'P', N'PC')
		ENDTEXT
		SQLEXEC(m.lnCnxHandle, m.lcSqlVerif ,"curresult")
		IF curresult.nb=0
			SQLEXEC(m.lnCnxHandle, m.lcProceduresCreate ,"curresult")
		ENDIF

		* la procdure ZAP historique fichiers
		TEXT TO lcProceduresCreate  NOSHOW TEXTMERGE

	/****** Object:  StoredProcedure [dbo].[Zap_Historique_Fichiers]    Script Date: 03/12/2017 15:48:02 ******/

	-- =============================================
	-- Author:		Michel
	-- Create date: 2017-07-18
	-- Description:	Purge en totalit l'historique de la table Fichiers
	-- =============================================
	CREATE PROCEDURE [dbo].[Zap_Historique_Fichiers]
	WITH EXECUTE AS SELF
	AS
	BEGIN
		-- SET NOCOUNT ON added to prevent extra result sets from
		-- interfering with SELECT statements.
		SET NOCOUNT ON;

	BEGIN TRAN;

	    ALTER TABLE Dbo.Fichiers SET(SYSTEM_VERSIONING = OFF);
	    EXEC (' delete from dbo.fichiers_historique');
	    ALTER TABLE Dbo.Fichiers SET(SYSTEM_VERSIONING = ON(HISTORY_TABLE = Dbo.Fichiers_Historique));
--	    DELETE FROM dbo.FoxTable_Fichiers_Projet WHERE FK_FoxTable IS NULL; --

	COMMIT;

	END

		ENDTEXT
		* on vrifie la non-existence avant de lancer l'exec
		TEXT TO lcSqlVerif  NOSHOW TEXTMERGE
		SELECT COUNT(*) as nb FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Zap_Historique_Fichiers]') AND type in (N'P', N'PC')
		ENDTEXT
		SQLEXEC(m.lnCnxHandle, m.lcSqlVerif ,"curresult")
		IF curresult.nb=0
			SQLEXEC(m.lnCnxHandle, m.lcProceduresCreate ,"curresult")
		ENDIF

		* on vrifie que c'est OK pour toutes les sproc

		TEXT TO lcSqlVerif  NOSHOW TEXTMERGE
		; WITH CTE
		     AS (
		     SELECT Name
		          FROM
		               [<<m.dbName>>].Sys.Objects
		          WHERE Name = N'Merge_FoxTable_depuis_DepotPourFox'
		     UNION ALL
		     SELECT Name
		          FROM
		               [<<m.dbName>>].Sys.Objects
		          WHERE Name = N'Merge_Fichiers_depuis_DepotPourFox'
		     UNION ALL
		     SELECT Name
		          FROM
		               [<<m.lcDbArchiveName>>].Sys.Objects
		          WHERE Name = N'Zap_Historique_FoxTable'
		     UNION ALL
		     SELECT Name
		          FROM
		               [<<m.lcDbArchiveName>>].Sys.Objects
		          WHERE Name = N'Zap_Historique_Fichiers'
		          )
		     SELECT COUNT(*) AS Nb
		          FROM
		               CTE;
		ENDTEXT

		SQLEXEC(m.lnCnxHandle, m.lcSqlVerif ,"curresult")
		IF curresult.nb=4
			m.llResult=.T.
		ELSE
			m.llResult=.F.
		ENDIF
	CATCH
		=AERROR(aErraray)
		MESSAGEBOX("plopProc")
	FINALLY
		USE IN SELECT("curresult")
	ENDTRY

	WAIT CLEAR
	RETURN m.llResult
ENDFUNC  && CreateProc

*************************************************

FUNCTION CreateViews AS Boolean
	LPARAMETERS dbName

	LOCAL llResult AS Boolean

	LOCAL lcDbArchiveName AS STRING
	m.lcDbArchiveName=ALLTRIM(m.dbName)+"_archive"

	TRY

		TEXT TO lcCreateViews NOSHOW TEXTMERGE
	USE [<<m.lcDbArchiveName>>] ;

	/****** Object:  View [dbo].[V_GetProjetFoxtable]    Script Date: 28/08/2017 15:17:25 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[V_GetProjetFoxtable]'))

	EXEC dbo.sp_executesql @statement = N'
	CREATE VIEW [dbo].[V_GetProjetFoxtable] AS
	   SELECT F.Fox_Nom_Table,
			F.Fox_Typetable,
			CAST(F.Fox_Chemin AS VARCHAR(254)) AS Fox_Chemin,
			F.Debut,
			F.Fin,
			COALESCE(P.Nom_Projet, '''')  AS Nom_Projet,
			COALESCE(P.Commentaire, '''') AS Commentaire,
			COALESCE(P.Niveau, '''') AS Niveau,
			CASE
			    WHEN F.Fin < ''9999-12-31 23:59:59.9999999''
                        AND NOT EXISTS (
						   SELECT * FROM Dbo.Foxtable FC
							       WHERE FC.PK = F.PK
					               )
			    THEN CAST(1 AS BIT)
			    ELSE CAST(0 AS BIT)
			END AS Deleted,
			F.PK AS PK_Nom_Table,
			 COALESCE(P.PK, -1) AS PK_Projet
	   FROM Dbo.Foxtable FOR SYSTEM_TIME ALL F
		   LEFT OUTER JOIN Dbo.FoxTable_Fichiers_Projet FP ON F.PK = FP.FK_Foxtable
		   LEFT OUTER JOIN Dbo.Projet P ON FP.FK_Projet = P.PK;
'

	/****** Object:  View [dbo].[V_GetProjetFichiers]    Script Date: 20/01/2018 19:13:06 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[V_GetProjetFichiers]'))
	EXEC dbo.sp_executesql @statement = N'

		CREATE VIEW [dbo].[V_GetProjetFichiers] AS
		   SELECT F.fichier_nom,
				F.fichier_type,
				CAST(F.fichier_chemin AS VARCHAR(254)) AS Fichier_Chemin,
				F.Debut,
				F.Fin,
				COALESCE(P.Nom_Projet, '''')  AS Nom_Projet,
				COALESCE(P.Commentaire, '''') AS Commentaire,
				COALESCE(P.Niveau, '''') AS Niveau,
				CASE
				    WHEN F.Fin < ''9999-12-31 23:59:59.9999999''
	                        AND NOT EXISTS (
							   SELECT * FROM Dbo.fichiers FC
								       WHERE FC.PK = F.PK
						               )
				    THEN CAST(1 AS BIT)
				    ELSE CAST(0 AS BIT)
				END AS Deleted,
				F.PK AS PK_Fichier_nom,
				 COALESCE(P.PK, -1) AS PK_Projet
		   FROM Dbo.fichiers FOR SYSTEM_TIME ALL F
			   LEFT OUTER JOIN dbo.FoxTable_Fichiers_Projet FP ON F.PK = FP.FK_Fichiers
			   LEFT OUTER JOIN Dbo.Projet P ON FP.FK_Projet = P.PK;


' 


	/****** Object:  View [dbo].[V_SetProjet2Foxtable]    Script Date: 14/09/2017 15:29:07 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[V_SetProjet2Foxtable]'))
	EXEC dbo.sp_executesql @statement = N'CREATE VIEW [dbo].[V_SetProjet2Foxtable]
	AS

	    WITH PU
		    AS (
		    -- les fichiers et leurs projets s''ils en ont
		    SELECT DISTINCT
				 F.Fox_Nom_Table,
				 F.Fox_Typetable,
				 CAST(F.Fox_Chemin AS VARCHAR(254)) AS Fox_Chemin,
				 COALESCE(P.Nom_Projet, '''') AS Nom_Projet,
				 COALESCE(P.Niveau, '''') AS Niveau
		    FROM Dbo.Foxtable FOR SYSTEM_TIME ALL F
			    LEFT OUTER JOIN Dbo.FoxTable_Fichiers_Projet FP ON F.PK = FP.FK_Foxtable
			    LEFT OUTER JOIN Dbo.Projet P ON FP.FK_Projet = P.PK)

		    SELECT fox_nom_table,
				 fox_typetable,
				 Fox_Chemin,
				 Nom_Projet,
				 Niveau
		    FROM PU
		    UNION ALL
		    -- les projets sans fichiers
		    SELECT '''' AS Fox_Nom_Table,
				 '''' AS Fox_Typetable,
				 '''' AS Fox_Chemin,
				 PA.nom_projet,
				 PA.niveau
		    FROM dbo.Projet PA
		    WHERE NOT EXISTS
		    (
			   SELECT *
			   FROM PU
			   WHERE PU.Niveau = PA.niveau
		    )

'

	/****** Object:  View [dbo].[V_SetProjet2Fichiers]    Script Date: 20/01/2018 12:18:52 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[V_SetProjet2Fichiers]'))
	EXEC dbo.sp_executesql @statement = N'
	CREATE VIEW [dbo].[V_SetProjet2Fichiers]
		AS

		    WITH PU
			    AS (
			    -- les fichiers et leurs projets s''ils en ont
			    SELECT DISTINCT
					 F.fichier_nom,
					 F.fichier_type,
					 CAST(F.fichier_chemin AS VARCHAR(254)) AS Fichier_Chemin,
					 COALESCE(P.Nom_Projet, '''') AS Nom_Projet,
					 COALESCE(P.Niveau, '''') AS Niveau
			    FROM Dbo.fichiers FOR SYSTEM_TIME ALL F
				    LEFT OUTER JOIN dbo.FoxTable_Fichiers_Projet FP ON F.PK = FP.FK_Fichiers
				    LEFT OUTER JOIN Dbo.Projet P ON FP.FK_Projet = P.PK)

			    SELECT fichier_nom,
					 fichier_type,
					 Fichier_Chemin,
					 Nom_Projet,
					 Niveau
			    FROM PU
			    UNION ALL
			    -- les projets sans fichiers
			    SELECT '''' AS fichier_nom,
					 '''' AS fichier_type,
					 '''' AS Fichier_Chemin,
					 PA.nom_projet,
					 PA.niveau
			    FROM dbo.Projet PA
			    WHERE NOT EXISTS
			    (
				   SELECT *
				   FROM PU
				   WHERE PU.Niveau = PA.niveau
			    )

' 


	/****** Object:  View [dbo].[V_SetVersion2Foxtable]    Script Date: 03/10/2017 14:57:28 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[V_SetVersion2Foxtable]'))
	EXEC dbo.sp_executesql @statement = N'

	CREATE VIEW [dbo].[V_SetVersion2Foxtable]
	AS
	     SELECT DISTINCT
	            V.Numero,
			  V.Debut,            
	            CAST(F.Fox_Chemin AS VARCHAR(254)) AS Fox_Chemin,
	            F.Fox_Nom_Table,
	            F.Fox_Typetable
	          FROM
	               Dbo.Version V
	               JOIN Dbo.Foxtable FOR SYSTEM_TIME ALL F
	                       ON V.Fk_Foxtable = F.Pk;
' 

	/****** Object:  View [dbo].[V_SetVersion2Fichiers]    Script Date: 06/01/2018 14:10:36 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[V_SetVersion2Fichiers]'))
	EXEC dbo.sp_executesql @statement = N'
		CREATE VIEW [dbo].[V_SetVersion2Fichiers]
		AS

		     SELECT DISTINCT
		            V.Numero,
				  V.Debut,            
		            CAST(F.fichier_chemin AS VARCHAR(254)) AS Fichier_Chemin,
		            F.fichier_nom,
		            F.fichier_type
		          FROM
		               Dbo.Version V
		               JOIN Dbo.fichiers FOR SYSTEM_TIME ALL F
		                       ON V.FK_fichiers = F.Pk;

' 



	/****** Object:  View [dbo].[V_GetVersionFoxtable]    Script Date: 24/09/2017 18:02:53 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[V_GetVersionFoxtable]'))
	EXEC dbo.sp_executesql @statement = N'
	CREATE view [dbo].[V_GetVersionFoxtable]
	AS
	    WITH V
		    AS (
		    SELECT Numero,
				 Fk_Foxtable,
				 Debut,
				 COALESCE(LEAD(Debut) OVER(PARTITION BY Fk_Foxtable ORDER BY Debut), ''9999-12-31 23:59:59.9999999'') AS Fin
			    FROM
				    Dbo.Version
			  )

	    SELECT F.Pk,
		   F.Fox_Nom_Table,
		   F.Fox_Typetable,
		   CAST(F.Fox_Chemin AS VARCHAR(254)) AS Fox_Chemin,
		   F.Debut,
		   F.Fin,
		   COALESCE(V.Numero,'''') as Numero,
		   V.Debut AS Vdeb,
		   V.Fin AS Vfin
		   FROM
			  Dbo.Foxtable FOR SYSTEM_TIME ALL F
			  LEFT OUTER JOIN V
					ON F.Pk = V.Fk_Foxtable
					    AND F.Debut >= V.Debut
					    AND F.Debut < V.Fin
' 


	/****** Object:  View [dbo].[V_GetVersionFichiers]    Script Date: 20/12/2017 10:59:03 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[V_GetVersionFichiers]'))
	EXEC dbo.sp_executesql @statement = N'
		CREATE view [dbo].[V_GetVersionFichiers]
		AS
		    WITH V
			    AS (
			    SELECT Numero,
					 Fk_Fichiers,
					 Debut,
					 COALESCE(LEAD(Debut) OVER(PARTITION BY Fk_fichiers ORDER BY Debut), ''9999-12-31 23:59:59.9999999'') AS Fin
				    FROM
					    Dbo.Version
				  )

		    SELECT F.Pk,
			   F.fichier_nom,
			   F.fichier_type,
			   CAST(F.fichier_chemin AS VARCHAR(254)) AS fichier_chemin,
			   F.Debut,
			   F.Fin,
			   COALESCE(V.Numero,'''') as Numero,
			   V.Debut AS Vdeb,
			   V.Fin AS Vfin
			   FROM
				  Dbo.fichiers FOR SYSTEM_TIME ALL F
				  LEFT OUTER JOIN V
						ON F.Pk = V.FK_fichiers
						    AND F.Debut >= V.Debut
						    AND F.Debut < V.Fin

'

		/****** Object:  View [dbo].[V_GetProjetTous]    Script Date: 21/01/2018 12:16:59 ******/
		SET ANSI_NULLS ON
		;
		SET QUOTED_IDENTIFIER ON
		;
		IF NOT EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[V_GetProjetTous]'))
		EXEC dbo.sp_executesql @statement = N'


			CREATE VIEW [dbo].[V_GetProjetTous] AS

			WITH 
			Vfox as 
			(
			   SELECT F.Fox_Nom_Table as Nom,
					F.Fox_Typetable as [Type],
					CAST(F.Fox_Chemin AS VARCHAR(254)) AS Chemin,
					F.Debut,
					F.Fin,
					COALESCE(P.Nom_Projet, '''')  AS Nom_Projet,
					COALESCE(P.Commentaire, '''') AS Commentaire,
					COALESCE(P.Niveau, '''') AS Niveau,
					CASE
					    WHEN F.Fin < ''9999-12-31 23:59:59.9999999''
		                        AND NOT EXISTS (
								   SELECT * FROM Dbo.Foxtable FC
									       WHERE FC.PK = F.PK
							               )
					    THEN CAST(1 AS BIT)
					    ELSE CAST(0 AS BIT)
					END AS Deleted,
					F.PK AS PK,
					COALESCE(P.PK, -1) AS PK_Projet
			   FROM Dbo.Foxtable FOR SYSTEM_TIME ALL F
				   LEFT OUTER JOIN dbo.FoxTable_Fichiers_Projet FP ON F.PK = FP.FK_Foxtable
				   LEFT OUTER JOIN Dbo.Projet P ON FP.FK_Projet = P.PK
			)

		,

		    Vfich as
		    (	   
			   SELECT F.fichier_nom as Nom,
				  F.fichier_type as [Type],
				  CAST(F.fichier_chemin AS VARCHAR(254)) AS Chemin,
				  F.Debut,
				  F.Fin,
				  COALESCE(P.Nom_Projet, '''')  AS Nom_Projet,
				  COALESCE(P.Commentaire, '''') AS Commentaire,
				  COALESCE(P.Niveau, '''') AS Niveau,
				  CASE
					 WHEN F.Fin < ''9999-12-31 23:59:59.9999999''
						AND NOT EXISTS (
								  SELECT * FROM Dbo.fichiers FC
										WHERE FC.PK = F.PK
									 )
					 THEN CAST(1 AS BIT)
					 ELSE CAST(0 AS BIT)
				  END AS Deleted,
				  F.PK AS PK,
				  COALESCE(P.PK, -1) AS PK_Projet
				  FROM Dbo.fichiers FOR SYSTEM_TIME ALL F
					  LEFT OUTER JOIN dbo.FoxTable_Fichiers_Projet FP ON F.PK = FP.FK_Fichiers
					  LEFT OUTER JOIN Dbo.Projet P ON FP.FK_Projet = P.PK)

			   SELECT Pk,
					 Nom,
					 [Type],
					 Chemin,
					 Debut,
					 Fin,
					 Deleted,
					 Pk_Projet,
					 Nom_Projet,
					 Commentaire,
					 Niveau
					 FROM
						Vfox
			   UNION ALL
			   SELECT Pk,
					 Nom,
					 [Type],
					 Chemin,
					 Debut,
					 Fin,
					 Deleted,
					 Pk_Projet,
					 Nom_Projet,
					 Commentaire,
					 Niveau
					 FROM
						Vfich;
				   


' 


		/****** Object:  View [dbo].[V_GetVersionTous]    Script Date: 20/12/2017 12:05:17 ******/
		SET ANSI_NULLS ON
		;
		SET QUOTED_IDENTIFIER ON
		;
			IF NOT EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[V_GetVersionTous]'))
			EXEC dbo.sp_executesql @statement = N'
		CREATE view [dbo].[V_GetVersionTous] as

		-- les fichiers fox
		WITH Vfox
		     AS (
		     SELECT Numero,
		            Fk_Foxtable,
		            Debut,
		            COALESCE(LEAD(Debut) OVER(PARTITION BY Fk_Foxtable ORDER BY Debut), ''9999-12-31 23:59:59.9999999'') AS Fin
		          FROM
		               Dbo.Version),

		     Vfoxa
		     AS (
		     SELECT F.Pk,
		            F.Fox_Nom_Table AS Nom,
		            F.Fox_Typetable AS [Type],
		            CAST(F.Fox_Chemin AS VARCHAR(254)) AS Chemin,
		            F.Debut,
		            F.Fin,
		            COALESCE(Vfox.Numero, '''') AS Numero,
		            Vfox.Debut AS Vdeb,
		            Vfox.Fin AS Vfin
		          FROM
		               Dbo.Foxtable FOR SYSTEM_TIME ALL F
		               LEFT OUTER JOIN Vfox
		                       ON F.Pk = Vfox.Fk_Foxtable
		                          AND F.Debut >= Vfox.Debut
		                          AND F.Debut < Vfox.Fin),

		-- les fichiers non-fox
		     Vfich
		     AS (
		     SELECT Numero,
		            Fk_Fichiers,
		            Debut,
		            COALESCE(LEAD(Debut) OVER(PARTITION BY Fk_Fichiers ORDER BY Debut), ''9999-12-31 23:59:59.9999999'') AS Fin
		          FROM
		               Dbo.Version),

		     Vficha
		     AS (
		     SELECT F.Pk,
		            F.Fichier_Nom AS Nom,
		            F.Fichier_Type AS [Type],
		            CAST(F.Fichier_Chemin AS VARCHAR(254)) AS Chemin,
		            F.Debut,
		            F.Fin,
		            COALESCE(Vfich.Numero, '''') AS Numero,
		            Vfich.Debut AS Vdeb,
		            Vfich.Fin AS Vfin
		          FROM
		               Dbo.Fichiers FOR SYSTEM_TIME ALL F
		               LEFT OUTER JOIN Vfich
		                       ON F.Pk = Vfich.Fk_Fichiers
		                          AND F.Debut >= Vfich.Debut
		                          AND F.Debut < Vfich.Fin)

		-- on assemble

		     SELECT Pk,
		            Nom,
		            [Type],
		            Chemin,
		            Debut,
		            Fin,
		            Numero,
		            Vdeb,
		            Vfin
		          FROM
		               Vfoxa
		     UNION ALL
		     SELECT Pk,
		            Nom,
		            [Type],
		            Chemin,
		            Debut,
		            Fin,
		            Numero,
		            Vdeb,
		            Vfin
		          FROM
		               Vficha					     

'



/****** Object:  Trigger [dbo].[T_IofDELETE_V_SetProjet2Foxtable]    Script Date: 14/09/2017 15:29:07 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'[dbo].[T_IofDELETE_V_SetProjet2Foxtable]'))
	EXEC dbo.sp_executesql @statement = N'
	-- =============================================
	-- Author:		Michel
	-- Create date: 2017-09-10
	-- Description:	en cas de suppression
	-- =============================================
	CREATE TRIGGER [dbo].[T_IofDELETE_V_SetProjet2Foxtable] ON [dbo].[V_SetProjet2Foxtable]
	INSTEAD OF DELETE
	AS
	     BEGIN
		-- SET NOCOUNT ON added to prevent extra result sets from
		-- interfering with SELECT statements.
	         SET NOCOUNT ON;

	  -- depuis deleted, trouver la PK de fox_table et celle de Projets
	  -- donc faire une jointure DELETED->FoxTable sur Fox_Nom_Table, Fox_TypeTable, Fox_Chemin et ->Projet sur Niveau
	         WITH ASUPPR
	              AS (
	              SELECT D.fox_nom_table,
	                     D.fox_typetable,
	                     D.Fox_Chemin,
	                     D.Nom_Projet,
	                     D.Niveau,
	                     T.PK AS PK_table,
	                     P.PK AS PK_projet
	              FROM DELETED D
	                   JOIN dbo.foxtable T ON D.fox_nom_table = T.fox_nom_table
	                                          AND D.fox_typetable = T.fox_typetable
	                                          AND D.fox_chemin = T.fox_chemin
	                   JOIN dbo.Projet P ON D.niveau = P.niveau)
	-- supprimer selon le rsultat de la jointure
	              DELETE dbo.FoxTable_Fichiers_Projet
	              FROM dbo.FoxTable_Fichiers_Projet
	                   JOIN ASUPPR ON FK_Projet = ASUPPR.PK_projet
	                                  AND FK_FoxTable = ASUPPR.PK_table;
	     END;


'
	/****** Object:  Trigger [dbo].[T_IofINSERT_V_SetProjet2Foxtable]    Script Date: 14/09/2017 15:29:07 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'[dbo].[T_IofINSERT_V_SetProjet2Foxtable]'))
	EXEC dbo.sp_executesql @statement = N'
	-- =============================================
	-- Author:		Michel
	-- Create date: 2017-09-11
	-- Description:	en cas d''insertion
	-- =============================================
	CREATE TRIGGER [dbo].[T_IofINSERT_V_SetProjet2Foxtable] ON [dbo].[V_SetProjet2Foxtable]
	INSTEAD OF INSERT
	AS
	     BEGIN
		-- SET NOCOUNT ON added to prevent extra result sets from
		-- interfering with SELECT statements.
	         SET NOCOUNT ON;

	  -- depuis Inserted, trouver la PK de fox_table et celle de Projets
	  -- donc faire une jointure INSERTED->FoxTable sur Fox_Nom_Table, Fox_TypeTable, Fox_Chemin et ->Projet sur Niveau
	         WITH T
	              AS (
	              SELECT PK,
	                     fox_nom_table,
	                     fox_typetable,
	                     fox_chemin
	              FROM dbo.foxtable FOR SYSTEM_TIME ALL)
			    ,

	              AINS
	              AS (
	              SELECT I.fox_nom_table,
	                     I.fox_typetable,
	                     I.Fox_Chemin,
	                     I.Nom_Projet,
	                     I.Niveau,
	                     T.PK AS PK_table,
	                     P.PK AS PK_projet
	              FROM INSERTED I
	                   JOIN T ON I.fox_nom_table = T.fox_nom_table
	                                          AND I.fox_typetable = T.fox_typetable
	                                          AND I.fox_chemin = T.fox_chemin
	                   JOIN dbo.Projet P ON I.niveau = P.niveau)

	-- insrer le rsultat de la jointure
	              INSERT INTO dbo.FoxTable_Fichiers_Projet
	              (FK_Projet,
	               FK_FoxTable
	              )
	                     SELECT DISTINCT 
	                            PK_projet,
	                            PK_table
	                     FROM AINS;
	     END;


'
	/****** Object:  Trigger [dbo].[T_IofUPDATE_V_SetProjet2Foxtable]    Script Date: 14/09/2017 15:29:07 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'[dbo].[T_IofUPDATE_V_SetProjet2Foxtable]'))
	EXEC dbo.sp_executesql @statement = N'
	-- =============================================
	-- Author:		Michel
	-- Create date: 2017-09-11
	-- Description:	sur update, uniquement si niveau est modifi
	-- =============================================
	CREATE TRIGGER [dbo].[T_IofUPDATE_V_SetProjet2Foxtable] ON [dbo].[V_SetProjet2Foxtable]
	INSTEAD OF UPDATE
	AS
	     BEGIN
		-- SET NOCOUNT ON added to prevent extra result sets from
		-- interfering with SELECT statements.
	         SET NOCOUNT ON;

	         IF UPDATE(Niveau)
	             BEGIN
	                 WITH T1
	                      AS (
	                      SELECT PK,
	                             fox_nom_table,
	                             fox_typetable,
	                             fox_chemin
	                      FROM dbo.foxtable FOR SYSTEM_TIME ALL)
					  ,

	                      ASSUPR
	                      AS (
	                      SELECT D.fox_nom_table,
	                             D.fox_typetable,
	                             D.Fox_Chemin,
	                             D.Nom_Projet,
	                             D.Niveau,
	                             T1.PK AS PK_table,
	                             P.PK AS PK_projet
	                      FROM DELETED D
	                           JOIN T1 ON D.fox_nom_table = T1.fox_nom_table
	                                      AND D.fox_typetable = T1.fox_typetable
	                                      AND D.fox_chemin = T1.fox_chemin
	                           JOIN dbo.Projet P ON D.niveau = P.niveau)
					  ,

	                      AINS
	                      AS (
	                      SELECT I.fox_nom_table,
	                             I.fox_typetable,
	                             I.Fox_Chemin,
	                             I.Nom_Projet,
	                             I.Niveau,
	                             T1.PK AS PK_table,
	                             P.PK AS PK_projet
	                      FROM INSERTED I
	                           JOIN T1 ON I.fox_nom_table = T1.fox_nom_table
	                                      AND I.fox_typetable = T1.fox_typetable
	                                      AND I.fox_chemin = T1.fox_chemin
	                           JOIN dbo.Projet P ON I.niveau = P.niveau)

					  ,

	                      result
	                      AS (
	                      SELECT DISTINCT
	                             assupr.PK_table,
	                             assupr.PK_projet AS OldFK_projet,
	                             AINS.PK_projet AS newFK_projet
	                      FROM AINS
	                           JOIN ASSUPR ON AINS.fox_nom_table = ASSUPR.fox_nom_table
	                                          AND ains.fox_typetable = assupr.fox_typetable
	                                          AND ains.Fox_Chemin = ASSUPR.Fox_Chemin)

					 UPDATE dbo.FoxTable_Fichiers_Projet
						 SET FK_Projet = newFK_projet
						   FROM FoxTable_Fichiers_Projet P
						    JOIN result ON P.FK_FoxTable = result.PK_table
									AND P.FK_Projet = result.OldFK_projet;

	         END; -- update(niveau)
		    ELSE
			  BEGIN
			  RAISERROR(N''La seule modification autorise est le projet de rattachement, dfini par son niveau'',16,1)
		    END ;
	     END;
'

	/****** Object:  Trigger [dbo].[T_IofDELETE_V_SetProjet2Fichiers]    Script Date: 20/01/2018 12:18:52 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'[dbo].[T_IofDELETE_V_SetProjet2Fichiers]'))
	EXEC dbo.sp_executesql @statement = N'-- =============================================
	-- Author:		Michel
	-- Create date: 2018-01-19
	-- Description:	en cas de suppression
	-- =============================================
	CREATE TRIGGER [dbo].[T_IofDELETE_V_SetProjet2Fichiers] ON [dbo].[V_SetProjet2Fichiers]
	INSTEAD OF DELETE
	AS
	         BEGIN
		-- SET NOCOUNT ON added to prevent extra result sets from
		-- interfering with SELECT statements.
	             SET NOCOUNT ON;

		  -- depuis deleted, trouver la PK de fox_table et celle de Projets
		  -- donc faire une jointure DELETED->Fichiers sur fichier_nom, fichier_type, fichier_chemin et ->Projet sur Niveau
	             WITH ASUPPR
	                  AS (
	                  SELECT D.Fichier_Nom,
	                         D.Fichier_Type,
	                         D.Fichier_Chemin,
	                         D.Nom_Projet,
	                         D.Niveau,
	                         F.PK AS PK_Fichier,
	                         P.PK AS PK_Projet
	                       FROM
	                            Deleted D
	                            JOIN Dbo.Fichiers F
	                                    ON D.Fichier_Nom = F.Fichier_Nom
	                                       AND D.Fichier_Type = F.Fichier_Type
	                                       AND D.Fichier_Chemin = F.Fichier_Chemin
	                            JOIN Dbo.Projet P
	                                    ON D.Niveau = P.Niveau)

		  	-- supprimer selon le rsultat de la jointure
	                  DELETE Dbo.Foxtable_Fichiers_Projet
	                       FROM Dbo.Foxtable_Fichiers_Projet
	                            JOIN ASUPPR
	                                    ON FK_Projet = PK_Projet
	                                       AND FK_Fichiers = ASUPPR.PK_Fichier;
	         END;
	' 
	;
	/****** Object:  Trigger [dbo].[T_IofINSERT_V_SetProjet2Fichiers]    Script Date: 20/01/2018 12:18:53 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'[dbo].[T_IofINSERT_V_SetProjet2Fichiers]'))
	EXEC dbo.sp_executesql @statement = N'-- =============================================
	-- Author:		Michel
	-- Create date: 2018-01-19
	-- Description:	en cas d''insertion
	-- =============================================
	CREATE TRIGGER [dbo].[T_IofINSERT_V_SetProjet2Fichiers] ON [dbo].[V_SetProjet2Fichiers]
	INSTEAD OF INSERT
	AS
	         BEGIN
		-- SET NOCOUNT ON added to prevent extra result sets from
		-- interfering with SELECT statements.
	             SET NOCOUNT ON;

		  -- depuis Inserted, trouver la PK de fox_table et celle de Projets
		  -- donc faire une jointure INSERTED->Fichiers sur fichier_nom, Fichier_type, Fichier_Chemin et ->Projet sur Niveau
	             WITH F
	                  AS (
	                  SELECT DISTINCT PK,
	                         Fichier_Nom,
	                         Fichier_Type,
	                         Fichier_Chemin
	                  FROM Dbo.Fichiers FOR SYSTEM_TIME ALL)
				   ,
	                  AINS
	                  AS (
	                  SELECT I.Fichier_Nom,
	                         I.Fichier_Type,
	                         I.Fichier_Chemin,
	                         I.Nom_Projet,
	                         I.Niveau,
	                         F.PK AS PK_Fichier,
	                         P.PK AS PK_Projet
	                       FROM
	                            INSERTED I
	                            JOIN F
	                                    ON I.Fichier_Nom = F.Fichier_Nom
	                                       AND I.Fichier_Type = F.Fichier_Type
	                                       AND I.Fichier_Chemin = F.Fichier_Chemin
	                            JOIN Dbo.Projet P
	                                    ON I.Niveau = P.Niveau)

		  -- insrer le rsultat de la jointure
		  
	                  INSERT INTO Dbo.Foxtable_Fichiers_Projet
				  (FK_Projet,
				   FK_Fichiers
				  )
	                         SELECT DISTINCT 
	                                PK_Projet,
	                                PK_Fichier
	                              FROM AINS;

	         END;
	' 
	;
	/****** Object:  Trigger [dbo].[T_IofUPDATE_V_SetProjet2Fichiers]    Script Date: 20/01/2018 12:18:53 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'[dbo].[T_IofUPDATE_V_SetProjet2Fichiers]'))
	EXEC dbo.sp_executesql @statement = N'-- =============================================
	-- Author:		Michel
	-- Create date: 2018-01-19
	-- Description:	sur update, uniquement si niveau est modifi
	-- =============================================
	CREATE TRIGGER [dbo].[T_IofUPDATE_V_SetProjet2Fichiers] 
	   ON  [dbo].[V_SetProjet2Fichiers] 
	   INSTEAD OF UPDATE
	AS 
	BEGIN
		-- SET NOCOUNT ON added to prevent extra result sets from
		-- interfering with SELECT statements.
		SET NOCOUNT ON;

	 	         IF UPDATE(Niveau)
		             BEGIN
		                 WITH T1
		                      AS (
		                      SELECT PK,
		                             fichier_nom,
		                             fichier_type,
		                             fichier_chemin
		                      FROM dbo.fichiers FOR SYSTEM_TIME ALL)
						  ,

		                      ASSUPR
		                      AS (
		                      SELECT D.fichier_nom,
		                             D.fichier_type,
		                             D.Fichier_Chemin,
		                             D.Nom_Projet,
		                             D.Niveau,
		                             T1.PK AS PK_fichier,
		                             P.PK AS PK_projet
		                      FROM DELETED D
		                           JOIN T1 ON D.fichier_nom = T1.fichier_nom
		                                      AND D.fichier_type = T1.fichier_type
		                                      AND D.Fichier_Chemin = T1.fichier_chemin
		                           JOIN dbo.Projet P ON D.niveau = P.niveau)
						  ,

		                      AINS
		                      AS (
		                      SELECT I.fichier_nom,
		                             I.fichier_type,
		                             I.Fichier_Chemin,
		                             I.Nom_Projet,
		                             I.Niveau,
		                             T1.PK AS PK_fichier,
		                             P.PK AS PK_projet
		                      FROM INSERTED I
		                           JOIN T1 ON I.fichier_nom = T1.fichier_nom
		                                      AND I.fichier_type = T1.fichier_type
		                                      AND I.Fichier_Chemin = T1.fichier_chemin
		                           JOIN dbo.Projet P ON I.niveau = P.niveau)

						  ,

		                      result
		                      AS (
		                      SELECT DISTINCT
		                             assupr.PK_fichier,
		                             assupr.PK_projet AS OldFK_projet,
		                             AINS.PK_projet AS newFK_projet
		                      FROM AINS
		                           JOIN ASSUPR ON AINS.fichier_nom = ASSUPR.fichier_nom
		                                          AND ains.fichier_type = assupr.fichier_type
		                                          AND ains.Fichier_Chemin = ASSUPR.Fichier_Chemin)

						 UPDATE dbo.FoxTable_Fichiers_Projet
							 SET FK_Projet = newFK_projet
							   FROM dbo.FoxTable_Fichiers_Projet P
							    JOIN result ON P.FK_Fichiers = result.PK_fichier
										AND P.FK_Projet = result.OldFK_projet;

		         END; -- update(niveau)
			    ELSE
				  BEGIN
				  RAISERROR(N''La seule modification autorise est le projet de rattachement, dfini par son niveau'',16,1)
			    END ;  

	END
	' 
	;

	/****** Object:  Trigger [dbo].[T_IofDELETE_V_SetVersion2Foxtable]    Script Date: 08/10/2017 19:27:37 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'[dbo].[T_IofDELETE_V_SetVersion2Foxtable]'))
	EXEC dbo.sp_executesql @statement = N'
	-- =============================================
	-- Author:		Michel
	-- Create date: 2017-10-05
	-- Description:	en cas de suppression
	-- =============================================
	CREATE TRIGGER [dbo].[T_IofDELETE_V_SetVersion2Foxtable] ON [dbo].[V_SetVersion2Foxtable]
	INSTEAD OF DELETE
	AS
	     BEGIN
		-- SET NOCOUNT ON added to prevent extra result sets from
		-- interfering with SELECT statements.
	         SET NOCOUNT ON;
	    -- on doit supprimer de la table Version celle qui correspond  un numero et une FK_foxtable
	    --il faut donc trouver les foxtables qui sont dans la DELETED

	         WITH ADEL
	              AS (
	              SELECT DISTINCT
	                     D.Numero,
	                     D.Debut,
	                     F.PK AS FK_foxtable
	              FROM DELETED D
	                   JOIN Dbo.Foxtable FOR SYSTEM_TIME ALL F ON D.Fox_Chemin = F.Fox_Chemin
	                                                              AND D.Fox_Nom_Table = F.Fox_Nom_Table
	                                                              AND D.Fox_Typetable = F.Fox_Typetable)
	              DELETE dbo.Version
	              FROM dbo.Version V
	                   JOIN ADEL ON V.FK_foxtable = ADEL.FK_foxtable
	                                AND V.debut = ADEL.Debut
	                                AND V.numero = ADEL.Numero;
	     END;
	' 
	;
	/****** Object:  Trigger [dbo].[T_IofINSERT_V_SetVersion2Foxtable]    Script Date: 08/10/2017 19:27:37 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'[dbo].[T_IofINSERT_V_SetVersion2Foxtable]'))
	EXEC dbo.sp_executesql @statement = N'
	-- =============================================
	-- Author:		Michel
	-- Create date: 2017-10-04
	-- Description:	en cas d''insertion
	-- =============================================
	CREATE TRIGGER [dbo].[T_IofINSERT_V_SetVersion2Foxtable] ON [dbo].[V_SetVersion2Foxtable]
	INSTEAD OF INSERT
	AS
	     BEGIN
		-- SET NOCOUNT ON added to prevent extra result sets from
		-- interfering with SELECT statements.
	         SET NOCOUNT ON;
	    -- on doit insrer dans la table Version un numeroID, un debut, une FK_foxtable
	    -- il faut donc trouver les foxtables qui sont dans l''INSERTED
	    -- si je reois le numero sous forme de 1.1.1 alors il faut le transformer, mais si je le reois sous forme d''un hierarchy, je peux insrer direct
	         WITH AINS
	              AS (
	              SELECT DISTINCT
	                     CASE
	                         WHEN TRY_CAST(I.numero AS HIERARCHYID) IS NULL
	                         THEN ''/''+LTRIM(RTRIM(REPLACE(I.numero, ''.'', ''/'')))+''/''
	                         ELSE I.numero
	                     END AS numero,
	                     I.Debut,
	                     F.PK AS FK_foxtable
	              FROM Inserted I
	                   JOIN Dbo.Foxtable FOR SYSTEM_TIME ALL F ON I.Fox_Chemin = F.Fox_Chemin
	                                                              AND I.Fox_Nom_Table = F.Fox_Nom_Table
	                                                              AND I.Fox_Typetable = F.Fox_Typetable)
	              
			    INSERT INTO dbo.Version
	              (numeroID,
	               debut,
	               FK_foxtable
	              )
	                     SELECT numero,
	                            COALESCE( Debut, ''1900-01-01''),
	                            FK_foxtable
	                     FROM AINS;
	     END;
	' 
	;
	/****** Object:  Trigger [dbo].[T_IofUPDATE_V_SetVersion2Foxtable]    Script Date: 08/10/2017 19:27:37 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'[dbo].[T_IofUPDATE_V_SetVersion2Foxtable]'))
	EXEC dbo.sp_executesql @statement = N'
	-- =============================================
	-- Author:		Michel
	-- Create date: 2017-10-08
	-- Description:	en cas d''update
	-- =============================================
	CREATE TRIGGER [dbo].[T_IofUPDATE_V_SetVersion2Foxtable] 
	   ON  [dbo].[V_SetVersion2Foxtable] 
	   INSTEAD OF UPDATE
	AS 
	BEGIN
		-- SET NOCOUNT ON added to prevent extra result sets from
		-- interfering with SELECT statements.
		SET NOCOUNT ON;
	    -- on va mettre  jour dans la table Version soit le numeroID, soit la date. la Fox_Table ne se modifie pas
	    IF UPDATE(Fox_chemin) OR UPDATE(Fox_Nom_Table) OR UPDATE(Fox_Typetable)
		   BEGIN
	   		  RAISERROR(N''Il est interdit de modifier le nom, le type, ou le dossier de la table'',16,1)
		   END;
	    ELSE
		   BEGIN
			  WITH 
				 ADEL
				   AS (
				   SELECT DISTINCT
						D.Numero,
						D.Debut,
						F.PK AS FK_foxtable
				   FROM DELETED D
					   JOIN Dbo.Foxtable FOR SYSTEM_TIME ALL F ON D.Fox_Chemin = F.Fox_Chemin
														 AND D.Fox_Nom_Table = F.Fox_Nom_Table
														 AND D.Fox_Typetable = F.Fox_Typetable)
	    ,
	             
				 AINS
				   AS (
				   SELECT DISTINCT
						CASE
						    WHEN TRY_CAST(I.numero AS HIERARCHYID) IS NULL
						    THEN ''/''+LTRIM(RTRIM(REPLACE(I.numero, ''.'', ''/'')))+''/''
						    ELSE I.numero
						END AS numero,
						I.Debut,
						F.PK AS FK_foxtable
				   FROM Inserted I
					   JOIN Dbo.Foxtable FOR SYSTEM_TIME ALL F ON I.Fox_Chemin = F.Fox_Chemin
														 AND I.Fox_Nom_Table = F.Fox_Nom_Table
														 AND I.Fox_Typetable = F.Fox_Typetable)
	    ,
			  RESULT
				 AS (
				SELECT ADEL.FK_foxtable AS FK_FoxTable,
					   ADEL.Numero AS OldNumero,
					   AINS.numero AS NewNumero,
					   ADEL.Debut AS OldDebut,
					   AINS.Debut AS NewDebut
				FROM ADEL
					   JOIN AINS ON ADEL.FK_foxtable = AINS.FK_foxtable 
				 )
			  
			  UPDATE dbo.Version
			    SET
				   numeroID = R.NewNumero,
				   debut = COALESCE(R.NewDebut,''1900-01-01'')
			  FROM dbo.Version V
				  JOIN RESULT R ON V.FK_foxtable = R.FK_FoxTable
							    AND V.numero = R.OldNumero
							    AND V.debut = R.OldDebut;												     
		   END; -- on peut faire l''update
	END
' 

/****** Object:  Trigger [dbo].[T_IofDELETE_V_SetVersion2Fichiers]    Script Date: 06/01/2018 14:10:36 ******/
SET ANSI_NULLS ON
;
SET QUOTED_IDENTIFIER ON
;
IF NOT EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'[dbo].[T_IofDELETE_V_SetVersion2Fichiers]'))
EXEC dbo.sp_executesql @statement = N'

	-- =============================================
	-- Author:		Michel
	-- Create date: 2018-01-05
	-- Description:	en cas de suppression
	-- =============================================
	CREATE TRIGGER [dbo].[T_IofDELETE_V_SetVersion2Fichiers] ON [dbo].[V_SetVersion2Fichiers]
	INSTEAD OF DELETE
	AS
	     BEGIN
		-- SET NOCOUNT ON added to prevent extra result sets from
		-- interfering with SELECT statements.
	         SET NOCOUNT ON;
	    -- on doit supprimer de la table Version celle qui correspond  un numero et une FK_Fichiers
	    --il faut donc trouver les Fichiers qui sont dans la DELETED

	         WITH ADEL
	              AS (
	              SELECT DISTINCT
	                     D.Numero,
	                     D.Debut,
	                     F.PK AS FK_Fichiers
	              FROM DELETED D
	                   JOIN Dbo.Fichiers FOR SYSTEM_TIME ALL F ON D.fichier_chemin = F.fichier_chemin
	                                                              AND D.fichier_nom = F.fichier_nom)
	              DELETE dbo.Version
	              FROM dbo.Version V
	                   JOIN ADEL ON V.FK_Fichiers = ADEL.FK_Fichiers
	                                AND V.debut = ADEL.Debut
	                                AND V.numero = ADEL.Numero;
	     END;
	
' 
	;
	/****** Object:  Trigger [dbo].[T_IofINSERT_V_SetVersion2Fichiers]    Script Date: 06/01/2018 14:10:36 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'[dbo].[T_IofINSERT_V_SetVersion2Fichiers]'))
	EXEC dbo.sp_executesql @statement = N'

		-- =============================================
		-- Author:		Michel
		-- Create date: 2018-01-05
		-- Description:	en cas d''insertion
		-- =============================================
		CREATE TRIGGER [dbo].[T_IofINSERT_V_SetVersion2Fichiers] ON [dbo].[V_SetVersion2Fichiers]
		INSTEAD OF INSERT
		AS
		     BEGIN
			-- SET NOCOUNT ON added to prevent extra result sets from
			-- interfering with SELECT statements.
		         SET NOCOUNT ON;
		    -- on doit insrer dans la table Version un numeroID, un debut, une FK_Fichiers
		    -- il faut donc trouver les Fichierss qui sont dans l''INSERTED
		    -- si je reois le numero sous forme de 1.1.1 alors il faut le transformer, mais si je le reois sous forme d''un hierarchy, je peux insrer direct
		         WITH AINS
		              AS (
		              SELECT DISTINCT
		                     CASE
		                         WHEN TRY_CAST(I.numero AS HIERARCHYID) IS NULL
		                         THEN ''/''+LTRIM(RTRIM(REPLACE(I.numero, ''.'', ''/'')))+''/''
		                         ELSE I.numero
		                     END AS numero,
		                     I.Debut,
		                     F.PK AS FK_Fichiers
		              FROM Inserted I
		                   JOIN Dbo.Fichiers FOR SYSTEM_TIME ALL F ON I.fichier_chemin = F.fichier_chemin
		                                                              AND I.fichier_nom = F.fichier_nom)
		              
				    INSERT INTO dbo.Version
		              (numeroID,
		               debut,
		               FK_Fichiers
		              )
		                     SELECT numero,
		                           COALESCE( Debut, ''1900-01-01''),
		                            FK_Fichiers
		                     FROM AINS;
		     END;
		
	' 
	;
	/****** Object:  Trigger [dbo].[T_IofUPDATE_V_SetVersion2Fichiers]    Script Date: 06/01/2018 14:10:36 ******/
	SET ANSI_NULLS ON
	;
	SET QUOTED_IDENTIFIER ON
	;
	IF NOT EXISTS (SELECT * FROM sys.triggers WHERE object_id = OBJECT_ID(N'[dbo].[T_IofUPDATE_V_SetVersion2Fichiers]'))
	EXEC dbo.sp_executesql @statement = N'

		-- =============================================
		-- Author:		Michel
		-- Create date: 2018-01-05
		-- Description:	en cas d''update
		-- =============================================
		CREATE TRIGGER [dbo].[T_IofUPDATE_V_SetVersion2Fichiers] 
		   ON  [dbo].[V_SetVersion2Fichiers] 
		   INSTEAD OF UPDATE
		AS 
		BEGIN
			-- SET NOCOUNT ON added to prevent extra result sets from
			-- interfering with SELECT statements.
			SET NOCOUNT ON;
		    -- on va mettre  jour dans la table Version soit le numeroID, soit la date. On ne modifie pas les infos de Fichiers
		    IF UPDATE(Fichier_Chemin) OR UPDATE(Fichier_Nom) OR UPDATE(Fichier_Type)
			   BEGIN
		   		  RAISERROR(N''Il est interdit de modifier le nom, le type, ou le dossier du fichier'',16,1)
			   END;
		    ELSE
			   BEGIN
				  WITH 
					 ADEL
					   AS (
					   SELECT DISTINCT
							D.Numero,
							D.Debut,
							F.PK AS FK_Fichiers
					   FROM DELETED D
						   JOIN Dbo.Fichiers FOR SYSTEM_TIME ALL F ON D.fichier_chemin = F.fichier_chemin
															 AND D.fichier_nom = F.fichier_nom)
		    ,
		             
					 AINS
					   AS (
					   SELECT DISTINCT
							CASE
							    WHEN TRY_CAST(I.numero AS HIERARCHYID) IS NULL
							    THEN ''/''+LTRIM(RTRIM(REPLACE(I.numero, ''.'', ''/'')))+''/''
							    ELSE I.numero
							END AS numero,
							I.Debut,
							F.PK AS FK_Fichiers
					   FROM Inserted I
						   JOIN Dbo.Fichiers FOR SYSTEM_TIME ALL F ON I.Fichier_Chemin = F.fichier_chemin
															 AND I.fichier_nom = F.fichier_nom)
		    ,
				  RESULT
					 AS (
					SELECT ADEL.FK_Fichiers AS FK_Fichiers,
						   ADEL.Numero AS OldNumero,
						   AINS.numero AS NewNumero,
						   ADEL.Debut AS OldDebut,
						   AINS.Debut AS NewDebut
					FROM ADEL
						   JOIN AINS ON ADEL.FK_Fichiers = AINS.FK_Fichiers 
					 )
				  
				  UPDATE dbo.Version
				    SET
					   numeroID = R.NewNumero,
					   debut = COALESCE(R.NewDebut,''1900-01-01'')
				  FROM dbo.Version V
					  JOIN RESULT R ON V.FK_Fichiers = R.FK_Fichiers
								    AND V.numero = R.OldNumero
								    AND V.debut = R.OldDebut;												     
			   END; -- on peut faire l''update
		END

' 

		ENDTEXT
		SQLEXEC(m.lnCnxHandle, m.lcCreateViews ,"curresult")

		* la vrification
		TEXT TO lcSqlVerif  NOSHOW TEXTMERGE
		SELECT count(*) as nbV FROM sys.views WHERE object_id in 
			( OBJECT_ID(N'[dbo].[V_GetProjetFoxtable]'),
			OBJECT_ID(N'[dbo].[V_GetProjetFichiers]'),
			OBJECT_ID(N'[dbo].[V_SetProjet2Foxtable]'),
			OBJECT_ID(N'[dbo].[V_SetProjet2Fichiers]'),
			OBJECT_ID(N'[dbo].[V_SetVersion2Foxtable]'),
			OBJECT_ID(N'[dbo].[V_SetVersion2Fichiers]'),
			OBJECT_ID(N'[dbo].[V_GetVersionFoxtable]'),
			OBJECT_ID(N'[dbo].[V_GetVersionFichiers]'),
			OBJECT_ID(N'[dbo].[V_GetVersionTous]')
			)
		ENDTEXT
		SQLEXEC(m.lnCnxHandle, m.lcSqlVerif ,"curresult")

		IF curresult.nbV=9
			TEXT TO lcSqlVerif   NOSHOW TEXTMERGE
		SELECT COUNT(*) AS NbT
		    FROM Sys.Triggers
		    WHERE object_id IN(
			   OBJECT_ID(N'[dbo].[T_IofDELETE_V_SetProjet2Foxtable]'),
			   OBJECT_ID(N'[dbo].[T_IofINSERT_V_SetProjet2Foxtable]'),
			   OBJECT_ID(N'[dbo].[T_IofUPDATE_V_SetProjet2Foxtable]'),
			   OBJECT_ID(N'[dbo].[T_IofDELETE_V_SetProjet2Fichiers]'),
			   OBJECT_ID(N'[dbo].[T_IofINSERT_V_SetProjet2Fichiers]'),
			   OBJECT_ID(N'[dbo].[T_IofUPDATE_V_SetProjet2Fichiers]'),
			   OBJECT_ID(N'[dbo].[T_IofDELETE_V_SetVersion2Foxtable]'),
			   OBJECT_ID(N'[dbo].[T_IofINSERT_V_SetVersion2Foxtable]'),
			   OBJECT_ID(N'[dbo].[T_IofUPDATE_V_SetVersion2Foxtable]'),
			   OBJECT_ID(N'[dbo].[T_IofDELETE_V_SetVersion2Fichiers]'),
			   OBJECT_ID(N'[dbo].[T_IofINSERT_V_SetVersion2Fichiers]'),
			   OBJECT_ID(N'[dbo].[T_IofUPDATE_V_SetVersion2Fichiers]')
		    );
			ENDTEXT
			SQLEXEC(m.lnCnxHandle, m.lcSqlVerif ,"curresult")
			IF curresult.nbT=12
				m.llResult = .T.
			ELSE
				m.llResult = .F.
			ENDIF

		ELSE
			m.llResult = .F.
		ENDIF

	CATCH
		=AERROR(aErraray)
		MESSAGEBOX("plopViews")
		ASSERT .f. MESSAGE "plop vues"
	FINALLY
		USE IN SELECT("curresult")
	ENDTRY

	WAIT CLEAR
	RETURN m.llResult
ENDFUNC && CreateViews
